using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client;
using NQueue.Entities;
using System.IO;
using NQueue.DataServices;
using NQueue.Client.LoggingStrategies;

namespace NQueue.BusinessServices.Jobs
{
    /// <summary>
    /// Invokes a job in its own application domain
    /// </summary>
    public class IsolatedJobInvoker
    {
        /// <summary>
        /// Executes the job in an application domain
        /// </summary>
        /// <param name="job">Job to execute</param>
        /// <param name="loggingStrategy">Logger for the job to write log events to</param>
        /// <returns>Returns the outcome of the job execution</returns>
        /// <remarks>
        /// The same job instance could be being run as part of more than one work item simultaneously
        /// </remarks>
        public ExecutionResults ExecuteJob(DequeuedWorkItem job, IExecutionLoggingStrategy loggingStrategy)
        {
            //Get the configuration for the current domain to copy to the new domain
            AppDomainSetup currentAppDomainSetup = AppDomain.CurrentDomain.SetupInformation;
			//Setup info for the new domain
            AppDomainSetup newAppDomainSetup = new AppDomainSetup();

            newAppDomainSetup.ApplicationBase = currentAppDomainSetup.ApplicationBase;
            newAppDomainSetup.PrivateBinPath = "bin;bin\\Debug;bin\\Release";   //Consider surfacing this as configuration
			
            //If a config file has been provided, use it to populate some of the setup
			if(job.JobConfigFile != null && job.JobConfigFile != String.Empty)
			{
                FileInfo configFile = new FileInfo(job.JobConfigFile);
				newAppDomainSetup.ConfigurationFile = configFile.FullName;
                newAppDomainSetup.PrivateBinPath += (";" + configFile.Directory.FullName);
			}
			else
			{
                //otherwise default to the current context
                newAppDomainSetup.ConfigurationFile = currentAppDomainSetup.ConfigurationFile;
			}


            //Create the new app domain
            string appDomainFriendlyName = job.JobName;
            AppDomain newDomain = AppDomain.CreateDomain(appDomainFriendlyName, AppDomain.CurrentDomain.Evidence, 
                newAppDomainSetup);

            //Create the execution context
            ExecutionContext executionContext = new ExecutionContext(loggingStrategy, job.InstanceTTLTimeSpan);

            try
            {

                //Create an instance of the job in the new domain
                Job remoteJob = (Job)newDomain.CreateInstanceAndUnwrap(
                    job.JobAssemblyStrongName, job.JobClassType);

                //execute the job
                ExecutionResults jobResult = remoteJob.ExecuteJob(job.JobParameters, executionContext);

                //mark the work item as completed
                DBWorkQueue.MarkWorkItemComplete(job.WorkItemID, jobResult);

                return jobResult;
            }
            finally
            {
                try
                {
                    System.Diagnostics.Trace.WriteLine("Attempting to unload app domain");
                    AppDomain.Unload(newDomain);
                    System.Diagnostics.Trace.WriteLine("App domain unloaded");
                }
                catch (CannotUnloadAppDomainException unloadExc)
                {
                    EventLogger.LogErrorEvent("Error unloading app domain for job", unloadExc);
                }
            }
        }
    }
}
