using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using NQueue.DataServices;
using NQueue.Client;
using NQueue.Client.LoggingStrategies;
using System.Diagnostics;

namespace NQueue.BusinessServices.Jobs
{
    /// <summary>
    /// Responsible for dequeuing jobs from the job queue.
    /// Once called the code drains the queue of appropriate jobs.  Jobs are executed synchronously
    /// </summary>
    /// <remarks>
    /// Dequeues jobs for the configured host only.
    /// Jobs are marked as being processed by the host instance this JobDequeuer object is running within
    /// </remarks>
    public class JobDequeuer
    {


        /// <summary>
        /// Runs against the work item queue for the configured host.
        /// All matching work items are dequeued and executed synchronously
        /// </summary>
        public static void Execute(int hostInstanceID)
        {
            //Name of the machine we're executing on - used to honour server affinity
            string machineName = System.Environment.MachineName;
            //Dequeue jobs from the work queue until no more remain
            DequeuedWorkItem newItem;
            int workCount = 0;
            while ((newItem = DBWorkQueue.DequeueWorkItem(hostInstanceID, machineName)) != null)
            {
                Trace.WriteLine(string.Format("Dequeued Job {0}, instance {1}", newItem.JobName, newItem.JobInstanceName));
                workCount++;

                //Execute the job
                ExecuteWorkItem(newItem);    
            }
            Trace.WriteLine(string.Format("Dequeued and attempted execution of {0} items from the work queue for host instance {1}", workCount, hostInstanceID));
            
        }

        /// <summary>
        /// Executes a dequeued work item
        /// </summary>
        /// <param name="itemToExecute">Work item to execute</param>
        /// <remarks>
        /// Work items marked as disabled are not executed.
        /// Work items marked as singletons are only executed if a mutex based on the 
        /// job instance name can be obtained.
        /// </remarks>
        private static void ExecuteWorkItem(DequeuedWorkItem itemToExecute)
        {
            Trace.WriteLine(string.Format("Executing work item {0}", itemToExecute.JobName));
            NQueueDatabaseLoggingStrategy logger = new NQueueDatabaseLoggingStrategy(itemToExecute.WorkItemID);

            //if the item is disabled, it should not be executed
            if (itemToExecute.JobInstanceState == JobInstanceStates.Disabled)
            {
                Trace.WriteLine("Job instance was disabled, not executing");
                logger.WriteInfo("Job instance was disabled, not executing");
                DBWorkQueue.MarkWorkItemComplete(itemToExecute.WorkItemID, ExecutionResults.Disabled);
                return;
            }

            //mutexmanager is IDisposable, releasing the mutex (if held) on dispose
            using (MutexManager mutexManager = new MutexManager())
            {
                //if the work item is marked as a singleton, we need to obtain a mutex
                //round the execution of the job, otherwise the job will be marked as a
                //duplicate
                if (itemToExecute.IsSingleton)
                {
                    logger.WriteInfo(string.Format("Executing on {0}.  Instance is a singleton.  Mutex name {1}, TTL {2}", Environment.MachineName, itemToExecute.JobInstanceName,
                        itemToExecute.InstanceTTLTimeSpan));
                    //try and obtain the mutex
                    if (!mutexManager.ObtainMutex(itemToExecute.JobInstanceName,
                        itemToExecute.InstanceTTLTimeSpan))
                    {
                        logger.WriteInfo(string.Format("Instance was marked as a singleton and the mutex could not be obtained, this processing instance will now end, marking as a duplicate (work item {0}).", itemToExecute.WorkItemID));
                        //couldn't obtain the mutex - mark as duplicate and end
                        DBWorkQueue.MarkWorkItemComplete(itemToExecute.WorkItemID,
                            ExecutionResults.Duplicate);
                        return;
                    }
                    else
                        logger.WriteInfo("Mutex obtained");
                }
                else
                {
                    logger.WriteInfo(string.Format("Executing on {0}.  Instance is not a singleton", Environment.MachineName));
                }
                

                //The isolated job invoker executes the job in its own app domain.
                IsolatedJobInvoker jobInvoker = new IsolatedJobInvoker();
                try
                {
                    ExecutionResults workResult = jobInvoker.ExecuteJob(itemToExecute, logger);
                    Trace.WriteLine(workResult);
                    logger.WriteInfo(string.Format("Executed job \"{0}\" without error.  Job result was {1}", itemToExecute.JobName, workResult));
                }
                catch (Exception exc)
                {
                    //mark the work item as failed
                    DBWorkQueue.MarkWorkItemComplete(itemToExecute.WorkItemID, ExecutionResults.Failure);
                    logger.WriteError("Error executing work item " + itemToExecute.JobName, exc);
                }
            }
        }
    }
}
