using System;
using System.Collections.Generic;
using System.Text;
using NQueue.DataServices;
using NQueue.Entities;
using NQueue.Client;
using System.Runtime.Remoting;
using System.IO;

namespace NQueue.BusinessServices.Jobs
{
    public class JobManager
    {
        /// <summary>
        /// Inserts a new job definition into the database
        /// </summary>
        /// <param name="jobName"></param>
        /// <param name="jobDescription"></param>
        /// <param name="jobAssemblyStrongName"></param>
        /// <param name="jobClassType"></param>
        /// <param name="jobConfigFile"></param>
        /// <returns>Job definition instance representing the newly inserted job</returns>
        public static void AddJobDefinition(string jobName, string jobDescription, string jobAssemblyStrongName, string jobClassType, string jobConfigFile)
        {
            DBJobDefinition.InsertJobDefinition(jobName, jobDescription, jobAssemblyStrongName, jobClassType, jobConfigFile);
        }

        /// <summary>
        /// Deletes a job definition from the database
        /// </summary>
        /// <param name="jobDefinitionID"></param>
        public static void DeleteJobDefinition(JobDefinition jobDefinition)
        {
            DBJobDefinition.DeleteJobDefinition(jobDefinition.JobDefinitionID);
        }

                /// <summary>
        /// Inserts a new job instance into the database
        /// </summary>
        /// <param name="jobDefinitionID"></param>
        /// <param name="jobInstanceName"></param>
        /// <param name="hostID"></param>
        /// <param name="instanceTTLTimeSpan"></param>
        /// <param name="jobParameters"></param>
        /// <returns>Returns a populated job instance object including the job instance ID (job name is not set)</returns>
        public static void InsertJobInstance(int jobDefinitionID, string jobInstanceName, int hostID, TimeSpan instanceTTLTimeSpan, object jobParameters, bool isSingleton)
        {
            string jobParametersString = SerialisationHelper.SerialiseObject(jobParameters);
            DBJobInstance.InsertJobInstance(jobDefinitionID, jobInstanceName, hostID, instanceTTLTimeSpan, jobParametersString, isSingleton);
        }

        /// <summary>
        /// Updates a job instance to a different state (e.g. disabled)
        /// </summary>
        /// <param name="jobInstanceID">ID of the job instance to update</param>
        /// <param name="jobInstanceState">New state for the job instance</param>
        public static void UpdateJobInstanceState(int jobInstanceID, JobInstanceStates jobInstanceState)
        {
            DBJobInstance.UpdateJobInstanceState(jobInstanceID, jobInstanceState);
        }


        /// <summary>
        /// Deletes a job instance from the database
        /// </summary>
        /// <param name="jobInstance"></param>
        public static void DeleteJobInstance(JobInstance jobInstance)
        {
            DBJobInstance.DeleteJobInstance(jobInstance.JobInstanceID);
        }


        public static Job GetJobObject(int jobDefinitionID)
        {
            JobDefinition jobDefinition = DBJobDefinition.GetJobDefinition(jobDefinitionID);
            ObjectHandle handle = Activator.CreateInstance(jobDefinition.JobAssemblyStrongName, jobDefinition.JobClassType);
            return (Job)handle.Unwrap();
        }
        
        public static bool CanJobInstanceExecuteOnServer(string jobInstanceName, string serverName)
        {
            JobInstance instance = DBJobInstance.GetJobInstance(jobInstanceName);
            return CanJobInstanceExecuteOnServer(instance.JobInstanceID, serverName);
        }
        /// <summary>
        /// Returns true if a job instance can execute right now on a given server - 
        /// i.e. is the Job associated with the instance assigned to a host that has 
        /// an instance on the target server
        /// </summary>
        /// <param name="jobInstanceID">ID of the job instance being queried</param>
        /// <param name="serverName">Server name</param>
        /// <returns></returns>
        public static bool CanJobInstanceExecuteOnServer(int jobInstanceID, string serverName)
        {
            serverName = serverName.Trim().ToLower();
            //get the job instance to get the host ID
            JobInstance jobInstance = DBJobInstance.GetJobInstance(jobInstanceID);
            //get all host instances associated with the job instance
            foreach (HostInstance hostInstance in DBHostInstance.GetAllHostInstances(jobInstance.HostID))
            {
                if (serverName == hostInstance.ServerName.Trim().ToLower())
                    return true;
            }
            return false;
        }

        public static int GetJobInstanceStateID(int jobInstanceID)
        {
            JobInstance jobInstance = DBJobInstance.GetJobInstance(jobInstanceID);
            return (int)jobInstance.JobInstanceState;
        }

        public static void EnqueueJobInstanceNow(int jobInstanceID)
        {
            EnqueueJobInstanceNow(jobInstanceID, null);
        }
        public static void EnqueueJobInstanceNow(int jobInstanceID, string serverAffinity)
        {
            DBWorkQueue.EnqueueWorkItem(jobInstanceID, serverAffinity, DateTime.Now);
        }


        public static void EnqueueJobInstanceNow(string jobInstanceName)
        {
            EnqueueJobInstanceNow(jobInstanceName, null);
        }
        public static void EnqueueJobInstanceNow(string jobInstanceName, string serverAffinity)
        {
            JobInstance instance = DBJobInstance.GetJobInstance(jobInstanceName);
            EnqueueJobInstanceNow(instance.JobInstanceID, serverAffinity);
        }

        /// <summary>
        /// Returns all job instances from the database
        /// </summary>
        /// <returns></returns>
        public static List<JobInstance> GetAllJobInstances()
        {
            return DBJobInstance.GetAllJobInstances();
        }
        /// <summary>
        /// Returns all job instances associated with a given job definition
        /// </summary>
        /// <param name="jobDefinitionID"></param>
        /// <returns></returns>
        public static List<JobInstance> GetAllJobInstances(int jobDefinitionID)
        {
            return DBJobInstance.GetAllJobInstances(jobDefinitionID);
        }

        /// <summary>
        /// Returns all job definitions in the database
        /// </summary>
        /// <returns></returns>
        public static List<JobDefinition> GetAllJobDefinitions()
        {
            return DBJobDefinition.GetAllJobDefinitions();
        }
        public static List<JobInstanceSchedule> GetSchedulesForInstance(JobInstance instance)
        {
            return DBJobInstance.GetJobInstanceSchedulesForJobInstance(instance.JobInstanceID);
        }
    }
}
