using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using System.ComponentModel;

namespace NQueue.BusinessServices.Scheduling.ScheduleProcessors
{
    public class DailyScheduleProcessor : ScheduleProcessor
    {
        public DailyScheduleProcessor() : base() { ; }
        public DailyScheduleProcessor(int scheduleTypeID, int hostInstanceID) : base (scheduleTypeID, hostInstanceID)
        {
            ;
        }
        /// <summary>
        /// Checks if the schedule is due to be run
        /// </summary>
        /// <param name="lastRunDateTime"></param>
        /// <param name="anticipatedNextRunDateTime"></param>
        /// <returns></returns>
        public override ScheduleProcessingDueResults IsScheduleTypeDue(DateTime? lastRunDateTime, DateTime? anticipatedNextRunDateTime)
        {
            //The daily processing schedule should be looked at once a day

            //if never run before, its due
            if (lastRunDateTime == null) return ScheduleProcessingDueResults.Due;

            //get the difference between now, and the date part ONLY of when last run
            TimeSpan timeSinceLastRun = DateTime.Now.Date.Subtract(((DateTime)lastRunDateTime).Date);
            double totalDaysSinceLastRun = timeSinceLastRun.TotalDays;
            //if the number of days is less than 1, the schedule has already been run today and is not due
            if (totalDaysSinceLastRun < 1) return ScheduleProcessingDueResults.NotDue;
            //if the number of days is greater than or equal to 2, it is overdue
            if (totalDaysSinceLastRun >= 2) return ScheduleProcessingDueResults.Overdue;
            //otherwise the schedule is just due
            return ScheduleProcessingDueResults.Due;
        }

        public override Type ParametersType
        {
            get { return typeof(DailyScheduleProcessorParameters); }
        }

        protected override List<WorkItemToEnqueue> ExecuteConcreteInstanceScheduleProcessing(JobInstanceSchedule schedule, object scheduleParametersObject)
        {
            DailyScheduleProcessorParameters instanceParameters = (DailyScheduleProcessorParameters)scheduleParametersObject;
            List<WorkItemToEnqueue> itemsToEnqueue = new List<WorkItemToEnqueue>();
            //process differently depending on the variant
            switch (instanceParameters.DailyProcessVariant)
            {
                //NOTE: This will only be called once a day as IsScheduleTypeDue returns "Due" only once a day
                case DailyScheduleProcessorParameters.DayVariant.EveryDay:
                    //if the start date has passed, and hasn't been run today
                    if (instanceParameters.StartDateAndTime.Date <= DateTime.Now
                        && (!schedule.InstanceScheduleLastRun.HasValue || schedule.InstanceScheduleLastRun.Value.Date < DateTime.Now.Date))
                        EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                    break;
                case DailyScheduleProcessorParameters.DayVariant.EveryWeekDay:
                    //if the start date has passed, and this is a weekday always enqueue one item today
                    if (instanceParameters.StartDateAndTime.Date <= DateTime.Now &&
                        IsWeekDay(DateTime.Now.DayOfWeek))
                        EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                    break;
                case DailyScheduleProcessorParameters.DayVariant.EveryNDays:
                    //if the start date has passed and the previous process date is N days or more ago, enqueue one item today
                    if (instanceParameters.StartDateAndTime.Date <= DateTime.Now)
                    {
                        if (schedule.InstanceScheduleLastRun != null && DateTime.Now.Subtract((DateTime)schedule.InstanceScheduleLastRun).TotalDays >= instanceParameters.EveryNDaysFrequency)
                        {
                            EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                        }
                    }
                    break;
                default:
                    throw new InvalidOperationException("no code path authored for DailyProcessVariant: " + instanceParameters.DailyProcessVariant);
            }
            return itemsToEnqueue;
        }

        private bool IsWeekDay(DayOfWeek dayOfWeek)
        {
            switch (dayOfWeek)
            {
                case DayOfWeek.Saturday:
                case DayOfWeek.Sunday:
                    return false;
                default:
                    return true;
            }
        }



        public override DateTime GetIndicativeNextRunDateTime()
        {
            //should run tomorrow ASAP
            return DateTime.Now.Date.AddDays(1);
        }
    }

    public class DailyScheduleProcessorParameters
    {
        public enum DayVariant
        {
            [Description("Every day")]
            EveryDay,
            [Description("Every week day")]
            EveryWeekDay,
            [Description("Every n days")]
            EveryNDays
        }

        private DayVariant _dailyProcessVariant;
        [Description("The variant of daily schedule this schedule is")]
        public DayVariant DailyProcessVariant
        {
            get { return _dailyProcessVariant; }
            set { _dailyProcessVariant = value; }
        }

        private int _everyNDaysFrequency;
        [Description("If the DayVariant is set to every N days, this describes the number of days")]
        public int EveryNDaysFrequency
        {
            get { return _everyNDaysFrequency; }
            set { _everyNDaysFrequency = value; }
        }

        private DateTime _startDateAndTime;
        [Description("Date before which processing should not start.  Time of the day processing should occur")]
        public DateTime StartDateAndTime
        {
            get { return _startDateAndTime; }
            set { _startDateAndTime = value; }
        }
    }
}
