using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using System.ComponentModel;

namespace NQueue.BusinessServices.Scheduling.ScheduleProcessors
{
    /// <summary>
    /// Implements a schedule processor based around monthly or day of month processing
    /// </summary>
    public class MonthlyScheduleProcessor : ScheduleProcessor
    {
        public MonthlyScheduleProcessor() : base() { ; }
        public MonthlyScheduleProcessor(int scheduleTypeID, int hostInstanceID)
            : base(scheduleTypeID, hostInstanceID)
        {
            ;
        }
        public override Type ParametersType
        {
            get { return typeof(MonthlyScheduleProcessorParameters); }
        }

        public override ScheduleProcessingDueResults IsScheduleTypeDue(DateTime? lastRunDateTime, DateTime? anticipatedNextRunDateTime)
        {
            //The monthly processing schedule should be looked at once a day and instances
            //enqueued for that day's run only

            //if never run before, its due
            if (lastRunDateTime == null) return ScheduleProcessingDueResults.Due;

            //get the difference between now, and the date part ONLY of when last run
            TimeSpan timeSinceLastRun = DateTime.Now.Date.Subtract(lastRunDateTime.Value.Date);
            double totalDaysSinceLastRun = timeSinceLastRun.TotalDays;
            //if the number of days is less than 1, the schedule has already been run today and is not due
            if (totalDaysSinceLastRun < 1) return ScheduleProcessingDueResults.NotDue;
            //if the number of days is greater than or equal to 2, it is overdue
            if (totalDaysSinceLastRun >= 2) return ScheduleProcessingDueResults.Overdue;
            //otherwise the schedule is just due
            return ScheduleProcessingDueResults.Due;
        }

        public override DateTime GetIndicativeNextRunDateTime()
        {
            //should run in 1 days time
            return DateTime.Now.Date.AddDays(1);
        }

        protected override List<WorkItemToEnqueue> ExecuteConcreteInstanceScheduleProcessing(NQueue.Entities.JobInstanceSchedule schedule, object scheduleParametersObject)
        {
            MonthlyScheduleProcessorParameters instanceParameters = (MonthlyScheduleProcessorParameters)scheduleParametersObject;
            List<WorkItemToEnqueue> itemsToEnqueue = new List<WorkItemToEnqueue>();
            //check the start date and time has passed and that we haven't already run once today
            if (!schedule.InstanceScheduleLastRun.HasValue || schedule.InstanceScheduleLastRun.Value.Date < DateTime.Now.Date)
            {
                if (instanceParameters.StartDateAndTime.Date <= DateTime.Now)
                {
                    switch (instanceParameters.MonthlyProcessVariant)
                    {
                        case MonthlyScheduleProcessorParameters.MonthlyVariant.DayOfMonth:
                            //what day of the month is today?
                            int todaysDay = DateTime.Now.Day;
                            //look through each day in the days to process - if this is one of those days, enqueue
                            foreach (int day in instanceParameters.DaysToProcess)
                            {
                                if (day == todaysDay)
                                    EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                                break;
                            }
                            break;
                        case MonthlyScheduleProcessorParameters.MonthlyVariant.EveryNMonths:
                            //if the instance schedule has never run, enqueue for today
                            if (!schedule.InstanceScheduleLastRun.HasValue)
                            {
                                EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                            }
                            else
                            {
                                //check if n or more months have passed since the instance schedule last ran
                                int monthsSinceLastRun = DateTime.Now.Month - schedule.InstanceScheduleLastRun.Value.Month;
                                if (monthsSinceLastRun >= instanceParameters.EveryNMonthsValue)
                                    EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                            }
                            break;
                        case MonthlyScheduleProcessorParameters.MonthlyVariant.FirstDayOfMonth:
                            //if this is the first day of the month, enqueue for today
                            if (DateTime.Now.Day == 1)
                                EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                            break;
                        case MonthlyScheduleProcessorParameters.MonthlyVariant.LastDayOfMonth:
                            //if today is the last day of the month, enqueue -->
                            //if the number of days in this month = the day of the month, enqueue
                            DateTime now = DateTime.Now;
                            if (DateTime.DaysInMonth(now.Year, now.Month) == now.Day)
                                EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                            break;
                        default:
                            throw new InvalidOperationException("no code path authored for MonthlyProcessVariant: " + instanceParameters.MonthlyProcessVariant);
                    }
                }
            }
            return itemsToEnqueue;
        }

        public class MonthlyScheduleProcessorParameters
        {
            public enum MonthlyVariant
            {
                DayOfMonth,
                EveryNMonths,
                FirstDayOfMonth,
                LastDayOfMonth
            }
            private MonthlyVariant _monthlyProcessVariant;
            [Description("The variant of monthly schedule this schedule instance is")]
            public MonthlyVariant MonthlyProcessVariant
            {
                get { return _monthlyProcessVariant; }
                set { _monthlyProcessVariant = value; }
            }

            private int _everyNMonthsValue;
            [Description("If MonthlyProcessVariant is set to EveryNMonths, this gives the value of n")]
            public int EveryNMonthsValue
            {
                get { return _everyNMonthsValue; }
                set { _everyNMonthsValue = value; }
            }

            private int[] _daysToProcess;
            [Description("If MonthlyProcessVariant is set to DayOfMonth, the job will run on every specified day of the month.  If a month doesn't have a day (e.g. 31 in February) it will never run")]
            public int[] DaysToProcess
            {
                get { return _daysToProcess; }
                set { _daysToProcess = value; }
            }

            private DateTime _startDateAndTime;
            [Description("Date and time before which processing should not start.")]
            public DateTime StartDateAndTime
            {
                get { return _startDateAndTime; }
                set { _startDateAndTime = value; }
            }

        }
    }
}
