using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using NQueue.DataServices;
using System.IO;
using System.Xml.Serialization;
using System.Diagnostics;

namespace NQueue.BusinessServices.Scheduling.ScheduleProcessors
{
    /// <summary>
    /// Base class for all schedule processors
    /// </summary>
    public abstract class ScheduleProcessor
    {
        private int _scheduleTypeID;
        private int _hostInstanceID;

        public ScheduleProcessor() { }
        public ScheduleProcessor(int scheduleTypeID, int hostInstanceID)
        {
            this._scheduleTypeID = scheduleTypeID;
            this._hostInstanceID = hostInstanceID;
        }

        /// <summary>
        /// Schedule type ID in the database for this schedule type
        /// </summary>
        public int ScheduleTypeID
        {
            get { return _scheduleTypeID; }
            set { _scheduleTypeID = value; }
        }
        /// <summary>
        /// Host instance ID for the host instance wanting to process this schedule type
        /// </summary>
        public int HostInstanceID
        {
            get { return _hostInstanceID; }
            set { _hostInstanceID = value; }
        }

        /// <summary>
        /// The implementor should provide a type containing custom parameters here.  This type
        /// is serialised and deserialised to the database when the job instance schedule is created or loaded
        /// </summary>
        /// <remarks>
        /// Return typeof(string) here to simply use a string to set or return parameters rather than a  
        /// structured type
        /// </remarks>
        public abstract Type ParametersType { get; }

        /// <summary>
        /// Checks if the schedule type is due to be run
        /// </summary>
        /// <param name="lastRunDateTime"></param>
        /// <param name="anticipatedNextRunDateTime"></param>
        /// <returns></returns>
        public abstract ScheduleProcessingDueResults IsScheduleTypeDue(DateTime? lastRunDateTime, DateTime? anticipatedNextRunDateTime);

        /// <summary>
        /// Gets a date time when this schedule type should next be run
        /// </summary>
        /// <returns></returns>
        public abstract DateTime GetIndicativeNextRunDateTime();

        /// <summary>
        /// Returns a list of work items that should be enqueued
        /// </summary>
        /// <returns></returns>
        protected abstract List<WorkItemToEnqueue> ExecuteConcreteInstanceScheduleProcessing(JobInstanceSchedule schedule, object scheduleParametersObject);


        /// <summary>
        /// Processes the schedule type
        /// </summary>
        /// <returns></returns>
        public bool ProcessSchedule(DateTime? lastProcessed)
        {
            //Attempt to lock the schedule type to be being processed by this host instance
            bool successfullyMarked =
                DBScheduling.MarkScheduleTypeAsBeingProcessed(ScheduleTypeID, lastProcessed, HostInstanceID);
            //if we failed to mark, return false and stop processing - some other host instance is processing this schedule now
            if (!successfullyMarked) return false;

            bool unmarkedScheduleType = false;  //whether we successfully released the mark on the schedule type just gained
            try
            {
                //get all job instances on this schedule type - we will process these
                List<JobInstanceSchedule> instanceSchedules =
                    DBJobInstance.GetJobInstanceSchedulesForScheduleType(ScheduleTypeID);

                Trace.WriteLine(string.Format("Found {0} instance schedules for schedule type ID {1}", instanceSchedules.Count, ScheduleTypeID));

                //for each job instance that should be scheduled by this schedule processor, 
                //call the concrete implementors implementation for the schedule.
                //The implementor returns a list of job instances that should be enqueued.
                //The lists from all calls are summed and inserted to the queue as one transaction later
                List<WorkItemToEnqueue> allWorkItemsToEnqueue = new List<WorkItemToEnqueue>();
                foreach (JobInstanceSchedule schedule in instanceSchedules)
                {
                    allWorkItemsToEnqueue.AddRange(
                        ExecuteConcreteInstanceScheduleProcessing(schedule,    //pass this work to the concrete implementor
                            DeserialiseParametersObject(schedule.ScheduleParameters))); //deserialise the parameters object for the implementor
                }

                Trace.WriteLine(string.Format("Found {0} work items to enqueue", allWorkItemsToEnqueue.Count));

                //Enqueue all work items (if there are any) and mark the schedule as processed within a transaction
                DBWorkQueue.TransactionallyEnqueueWorkScheduleListAndMarkComplete(allWorkItemsToEnqueue, instanceSchedules,
                    ScheduleTypeID, lastProcessed, GetIndicativeNextRunDateTime(), HostInstanceID);
                unmarkedScheduleType = true;
            }
            finally
            {
                if (!unmarkedScheduleType)
                { 
                    //didn't manage to complete processing - unmark the schedule type
                    //so that another process can try
                    DBScheduling.RemoveScheduleTypeProcessingMark(ScheduleTypeID, HostInstanceID);
                }
            }

            //schedule processed
            return true;
        }

        protected static void EnqueueItemForToday(JobInstanceSchedule schedule, DateTime startDateAndTime, List<WorkItemToEnqueue> itemsToEnqueue)
        {
            itemsToEnqueue.Add(
                new WorkItemToEnqueue(schedule.JobInstanceID,
                    DateTime.Now.Date + startDateAndTime.TimeOfDay));
            schedule.JobInstanceScheduleWasJustProcessed = true;
        }

        /// <summary>
        /// Deserialises an XML serialised parameters object to an instance of the given type
        /// </summary>
        /// <param name="serialisedParameterInformation"></param>
        /// <returns></returns>
        public object DeserialiseParametersObject(string serialisedParameterInformation)
        {
            //if the parameters type is just a string, no need to deserialise
            if (ParametersType == typeof(string))
            {
                return serialisedParameterInformation;
            }
            else
            {
                //deserialise the parameter object
                byte[] buffer = Encoding.UTF8.GetBytes(serialisedParameterInformation);
                MemoryStream stringAsStream = new MemoryStream(buffer, 0, buffer.Length, true, true);
                XmlSerializer serializer = new XmlSerializer(ParametersType);
                return serializer.Deserialize(stringAsStream);
            }
        }

        /// <summary>
        /// Gets an instance of the parameters type in the default state
        /// </summary>
        /// <returns></returns>
        public object GetBlankParametersObject()
        {
            if (ParametersType == typeof(string)) return "";
            else return Activator.CreateInstance(ParametersType);
        }
    }
}
