using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using System.ComponentModel;

namespace NQueue.BusinessServices.Scheduling.ScheduleProcessors
{
    /// <summary>
    /// Implements a schedule processor based around weekly or day of week processing
    /// </summary>
    public class WeeklyScheduleProcessor : ScheduleProcessor
    {
        public WeeklyScheduleProcessor() : base() { ; }
        public WeeklyScheduleProcessor(int scheduleTypeID, int hostInstanceID)
            : base(scheduleTypeID, hostInstanceID)
        {
            ;
        }
        public override Type ParametersType
        {
            get { return typeof(WeeklyScheduleProcessorParameters); }
        }

        public override ScheduleProcessingDueResults IsScheduleTypeDue(DateTime? lastRunDateTime, DateTime? anticipatedNextRunDateTime)
        {
            //The weekly processing schedule should be looked at once a day and instances
            //enqueued for that day's run only

            //if never run before, its due
            if (lastRunDateTime == null) return ScheduleProcessingDueResults.Due;

            //get the difference between now, and the date part ONLY of when last run
            TimeSpan timeSinceLastRun = DateTime.Now.Date.Subtract(lastRunDateTime.Value.Date);
            double totalDaysSinceLastRun = timeSinceLastRun.TotalDays;
            //if the number of days is less than 1, the schedule has already been run today and is not due
            if (totalDaysSinceLastRun < 1) return ScheduleProcessingDueResults.NotDue;
            //if the number of days is greater than or equal to 2, it is overdue
            if (totalDaysSinceLastRun >= 2) return ScheduleProcessingDueResults.Overdue;
            //otherwise the schedule is just due
            return ScheduleProcessingDueResults.Due;
        }

        public override DateTime GetIndicativeNextRunDateTime()
        {
            //should run in 1 days time
            return DateTime.Now.Date.AddDays(1);
        }

        protected override List<WorkItemToEnqueue> ExecuteConcreteInstanceScheduleProcessing(NQueue.Entities.JobInstanceSchedule schedule, object scheduleParametersObject)
        {
            WeeklyScheduleProcessorParameters instanceParameters = (WeeklyScheduleProcessorParameters)scheduleParametersObject;
            List<WorkItemToEnqueue> itemsToEnqueue = new List<WorkItemToEnqueue>();
            //check the start date and time has passed and that we haven't already run once today
            if (!schedule.InstanceScheduleLastRun.HasValue || schedule.InstanceScheduleLastRun.Value.Date < DateTime.Now.Date)
            {
                if (instanceParameters.StartDateAndTime.Date <= DateTime.Now)
                {
                    switch (instanceParameters.WeeklyProcessVariant)
                    {
                        case WeeklyScheduleProcessorParameters.WeeklyVariant.DayOfWeek:
                            foreach (DayOfWeek day in instanceParameters.DaysToProcess)
                            {
                                //look at all the days specified matching against the day today
                                if (day == DateTime.Now.DayOfWeek)
                                {
                                    EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                                }
                            }
                            break;
                        case WeeklyScheduleProcessorParameters.WeeklyVariant.EveryNWeeks:
                            //if its never been run before, enqueue
                            if (!schedule.InstanceScheduleLastRun.HasValue)
                            {
                                EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                            }
                            else
                            {
                                //enqueue if the last run date and time was >= N weeks ago
                                if (DateTime.Now.Subtract(schedule.InstanceScheduleLastRun.Value).TotalDays >=
                                    instanceParameters.EveryNWeeksValue * 7)
                                {
                                    EnqueueItemForToday(schedule, instanceParameters.StartDateAndTime, itemsToEnqueue);
                                }
                            }
                            break;
                        default:
                            throw new InvalidOperationException("no code path authored for WeeklyProcessVariant: " + instanceParameters.WeeklyProcessVariant);
                    }
                }
            }
            return itemsToEnqueue;
        }

        public class WeeklyScheduleProcessorParameters
        {
            public enum WeeklyVariant
            {
                DayOfWeek,
                EveryNWeeks
            }
            private WeeklyVariant _weeklyProcessVariant;
            [Description("The variant of weekly schedule this schedule instance is")]
            public WeeklyVariant WeeklyProcessVariant
            {
                get { return _weeklyProcessVariant; }
                set { _weeklyProcessVariant = value; }
            }

            private int _everyNWeeksValue;
            [Description("If WeeklyProcessVariant is set to EveryNWeeks, this gives the value of n")]
            public int EveryNWeeksValue
            {
                get { return _everyNWeeksValue; }
                set { _everyNWeeksValue = value; }
            }

            private DayOfWeek[] _daysToProcess;
            [Description("If WeeklyProcessVariant is set to DayOfWeek, the job will run on every specified day of the week")]
            public DayOfWeek[] DaysToProcess
            {
                get { return _daysToProcess; }
                set { _daysToProcess = value; }
            }

            private DateTime _startDateAndTime;
            [Description("Date and time before which processing should not start.")]
            public DateTime StartDateAndTime
            {
                get { return _startDateAndTime; }
                set { _startDateAndTime = value; }
            }

        }
    }
}
