using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using NQueue.DataServices;
using NQueue.BusinessServices.Scheduling.ScheduleProcessors;
using System.Runtime.Remoting;
using System.Diagnostics;
using NQueue.Client;
namespace NQueue.BusinessServices.Scheduling
{

    /// <summary>
    /// Scheduling class - examines all schedule types on a periodic basis and attempts to load and
    /// execute each type that's due to be processed
    /// </summary>
    public class Scheduler
    {


        /// <summary>
        /// This method should be called periodically, e.g. every minute.
        /// All schedule processors are loaded and asked if the schedule type is due.  If due, the 
        /// processor is executed and all work items the scheduler deems appropriate are enqueued.
        /// </summary>
        public static void CheckAndExecuteSchedules(int hostInstanceID)
        {
            Trace.WriteLine(string.Format("Checking schedules for host instance ID {0}", hostInstanceID.ToString()));

            //get all schedule types
            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            if (schedules.Count == 0) throw new InvalidOperationException("No schedule types defined");
            
            //iterate over each schedule type and load the appropriate processor
            foreach (ScheduleType schedule in schedules)
            {
                //don't do anything with the schedule type if another host is processing it
                if (schedule.ProcessingHostInstanceID != null) continue;
                //create an instance of the relevant schedule processor
                ScheduleProcessor processor;
                //create an instance of the relevant processor object for the schedule type being processed
                processor = CreateScheduleProcessor(schedule);
                processor.ScheduleTypeID = schedule.ScheduleTypeID;
                processor.HostInstanceID = hostInstanceID;
                ScheduleProcessingDueResults results = processor.IsScheduleTypeDue(schedule.LastProcessed, schedule.NextProcessDue);
                Trace.WriteLine(string.Format("Processor {0} indicates schedule is {1}", schedule.ScheduleTypeClassName, results));
                switch (results)
                {
                    case ScheduleProcessingDueResults.NotDue:
                        //do nothing.
                        break;
                    case ScheduleProcessingDueResults.Overdue:
                        EventLogger.LogInformationEvent(string.Format("Schedule run for schedule type {0} was overdue and will now run", schedule.ScheduleTypeName));
                        processor.ProcessSchedule(schedule.LastProcessed);                            
                        break;
                    case ScheduleProcessingDueResults.Due:
                        processor.ProcessSchedule(schedule.LastProcessed);
                        break;
                    default:
                        break;
                }
            }
        }

        private static ScheduleProcessor CreateScheduleProcessor(ScheduleType schedule)
        {
            try
            {

                ObjectHandle handle = Activator.CreateInstance(schedule.ScheduleTypeStrongName, schedule.ScheduleTypeClassName);
                return (ScheduleProcessor) handle.Unwrap();
            }
            catch (Exception exc)
            {
                EventLogger.LogErrorEvent(string.Format("Fatal Error: Error activating instance of the schedule processor for schedule type \"{0}\", class {1} in {2}", schedule.ScheduleTypeName, schedule.ScheduleTypeClassName, schedule.ScheduleTypeStrongName), exc);
                throw;
            }                    

        }


    }
}
