using System;
using System.Collections.Generic;
using System.Text;
using NQueue.DataServices;
using NQueue.Entities;
using System.ServiceProcess;

namespace NQueue.BusinessServices.System
{
    /// <summary>
    /// Capable of starting\stopping\restarting all host instances
    /// as one operation
    /// </summary>
    public class HostInstanceController
    {
        /// <summary>
        /// Delegate to support information message
        /// </summary>
        /// <param name="message">Message being delivered</param>
        /// <param name="progress">Progress through all work</param>
        /// <param name="size">Size of the work</param>
        public delegate void InformationMessageDelegate(string message, int progress, int size);
        /// <summary>
        /// Raised when an informational event occurs
        /// </summary>
        public event InformationMessageDelegate InformationMessage;

        public delegate void FinishedDelegate();
        /// <summary>
        /// Raised when the controller has finished processing
        /// </summary>
        public event FinishedDelegate Finished;
        
        private ServiceControlCommands _command;
        public HostInstanceController(ServiceControlCommands command)
        {
            this._command = command;
        }

        public void ControlHostInstances()
        {
            switch (this._command)
            {
                case ServiceControlCommands.Stop:
                    ControlHostInstances(true, false);
                    break;
                case ServiceControlCommands.Start:
                    ControlHostInstances(false, true);
                    break;
                case ServiceControlCommands.Restart:
                    ControlHostInstances(true, true);
                    break;
            }
        }


        /// <summary>
        /// Stops and\or restarts all host instances
        /// </summary>
        /// <param name="doStop">Stops all host instances if set to true</param>
        /// <param name="doStart">Starts all host instances if set to true</param>
        private void ControlHostInstances(bool doStop, bool doStart)
        {
            try
            {
                ServiceController serviceController = new ServiceController();

                //get all host instances
                List<HostInstance> hostInstances = DBHostInstance.GetAllHostInstances();
                int progress = 0;
                int workSize = hostInstances.Count;
                if (doStop && doStart) workSize *= 2;

                //do all stops first
                if (doStop)
                {
                    foreach (HostInstance instance in hostInstances)
                    {
                        Info("Host {0} on {1}", progress, workSize, instance.HostName, instance.ServerName);
                        serviceController.ServiceName = SystemManager.GetNQueueServiceName(instance);
                        serviceController.MachineName = instance.ServerName;
                        switch (serviceController.Status)
                        {
                            case ServiceControllerStatus.Running:
                                Info("Stopping service {0}", progress, workSize, serviceController.ServiceName);
                                serviceController.Stop();
                                serviceController.WaitForStatus(ServiceControllerStatus.Stopped);
                                progress++;
                                Info("Stopped service", progress, workSize);
                                break;
                            case ServiceControllerStatus.Stopped:
                                progress++;
                                Info("Service already stopped", progress, workSize);
                                break;
                            default:
                                progress++;
                                Info("Status of service is \"{0}\", no action taken", progress, workSize, serviceController.Status);
                                break;
                        }
                    }
                }

                //now do starts
                if (doStart)
                {
                    foreach (HostInstance instance in hostInstances)
                    {
                        Info("Host {0} on {1}", progress, workSize, instance.HostName, instance.ServerName);
                        serviceController.ServiceName = SystemManager.GetNQueueServiceName(instance);
                        serviceController.MachineName = instance.ServerName;
                        switch (serviceController.Status)
                        {
                            case ServiceControllerStatus.Stopped:
                                Info("Starting service {0}", progress, workSize, serviceController.ServiceName);
                                serviceController.Start();
                                serviceController.WaitForStatus(ServiceControllerStatus.Running);
                                progress++;
                                Info("Started service", progress, workSize);
                                break;
                            case ServiceControllerStatus.Running:
                                progress++;
                                Info("Service already running", progress, workSize);
                                break;
                            default:
                                progress++;
                                Info("Status of service is \"{0}\", no action taken", progress, workSize, serviceController.Status);
                                break;
                        }
                    }
                }
            }
            finally
            {
                Finish();
            }
        }

        private void Info(string infoMessage, int progress, int size, params object[] args)
        {
            if (InformationMessage != null)
                InformationMessage(string.Format(infoMessage, args), progress, size);
        }

        private void Finish()
        {
            if (Finished != null) Finished();
        }

    }

    public enum ServiceControlCommands
    {
        Stop,
        Start,
        Restart
    }
}
