using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Data;


namespace NQueue.Client.DataServices
{
    /// <summary>
    /// Data layer for handling mutex acquisition for the shared locking service
    /// that client jobs can use
    /// </summary>
    public class DBMutex
    {
        /// <summary>
        /// Expires any outstanding mutexes in the mutex table
        /// </summary>
        public static void ExpireMutexes(string connectionString)
        {
            DBCommon.ExecuteParameterlessNonQuery(new SqlConnection(connectionString), "NQueue_ExpireMutexes");
        }

        /// <summary>
        /// Attempts to obtain a mutex for a given unique mutext name
        /// </summary>
        /// <param name="connectionString">Connection string to use to log the information</param>
        /// <param name="mutexName">Unique name for the mutex</param>
        /// <param name="serverName">Server name of the server requesting the mutex</param>
        /// <param name="mutexTTL">Time to live of the mutex - after this time other processes can take the mutex</param>
        /// <param name="mutexID">If the mutex was obtained, contains the mutex ID.  Otherwise will be null</param>
        /// <returns>Returns true if the mutex was successfully obtained</returns>
        /// <remarks>
        /// Connection string is passed into this call as the work logger is mobile across
        /// app domains and cannot rely on having app.config entries present when being called
        /// from a job implementation.
        /// </remarks>
        public static bool ObtainMutex(string connectionString, string mutexName, string serverName, TimeSpan mutexTTL, out int? mutexID)
        {
            using (SqlConnection cnn = new SqlConnection(connectionString))
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_ObtainMutex");
                cmd.Parameters.Add("@MutexName", SqlDbType.VarChar, 500).Value = mutexName;
                cmd.Parameters.Add("@ServerName", SqlDbType.VarChar, 256).Value = serverName;
                cmd.Parameters.Add("@ExpiryDate", SqlDbType.DateTime).Value = DateTime.Now.Add(mutexTTL);
                SqlParameter paramReturn = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();
                int returnValue = (Int32) paramReturn.Value;
                if (returnValue == 0)
                {
                    //did not obtain the mutex
                    mutexID = null;
                    return false;
                }
                else
                {
                    mutexID = returnValue;
                    return true;
                }
            }
        }
        /// <summary>
        /// Releases a held mutex 
        /// </summary>
        /// <param name="connectionString">Connection string to use to log the information</param>
        /// <param name="mutexID">ID of the mutex previously obtained</param>
        /// <param name="serverName">Servername releasing the mutex</param>
        /// <remarks>The mutex must have previously been held by the server specified in serverName</remarks>
        public static void ReleaseMutex(string connectionString, int mutexID, string serverName)
        {
            ExecuteReleaseMutexProcedure(connectionString, mutexID, serverName, "NQueue_ReleaseMutex");
        }
        /// <summary>
        /// Releases a held mutex 
        /// </summary>
        /// <param name="connectionString">Connection string to use to log the information</param>
        /// <param name="mutexID">ID of the mutex previously obtained</param>
        /// <param name="serverName">Servername releasing the mutex</param>
        /// <remarks>The mutex must have previously been held by the server specified in serverName</remarks>
        public static void DeleteMutex(string connectionString, int mutexID, string serverName)
        {
            ExecuteReleaseMutexProcedure(connectionString, mutexID, serverName, "NQueue_DeleteMutex");
        }

        private static void ExecuteReleaseMutexProcedure(string connectionString, int mutexID, string serverName, string procedureName)
        {
            using (SqlConnection cnn = new SqlConnection(connectionString))
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, procedureName);
                cmd.Parameters.Add("@MutexID", SqlDbType.Int).Value = mutexID;
                cmd.Parameters.Add("@ServerName", SqlDbType.VarChar, 256).Value = serverName;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }



    }
}
