using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using NQueue.Client.LoggingStrategies;

namespace NQueue.Client
{
    /// <summary>
    /// Base class for all valid NQueue jobs
    /// </summary>
    public abstract class Job : MarshalByRefObject
    {
        private string _configFilePath;
        private object _parametersObject;

        public Job() { ;}

        /// <summary>
        /// Alllows the concrete implementor to execute the job
        /// </summary>
        /// <returns>Returns an ExecutionResults enumeration value indicating the outcome of the job processing</returns>
        protected abstract ExecutionResults ExecuteJobImplementation(ExecutionContext context);

        /// <summary>
        /// The concrete implementor should return a short description of the job here
        /// </summary>
        /// <returns></returns>
        protected abstract string JobTypeName {get;}
        
        /// <summary>
        /// Executes the job
        /// </summary>
        /// <returns></returns>
        public ExecutionResults ExecuteJob(string serialisedParameterInformation, ExecutionContext context)
        {
            //if the parameters type is just a string, no need to deserialise
            if (ParametersType == typeof(string))
            {
                ParametersObject = serialisedParameterInformation;
            }
            else
            {
                //deserialise the parameter object
                ParametersObject = GetDeserialisedParametersObject(serialisedParameterInformation);
            }
            context.Logger.WriteInfo("Starting job " + JobTypeName);
            //execute the job
            return ExecuteJobImplementation(context);
        }

        /// <summary>
        /// The implementor should provide a type containing custom parameters here.  This type
        /// is serialised and deserialised to the database when the job instance is created or loaded
        /// </summary>
        /// <remarks>
        /// Return typeof(string) here to simply use a string to set or return parameters rather than a  
        /// structured type
        /// </remarks>
        public abstract Type ParametersType { get; }

        /// <summary>
        /// If an explict configuration file path has been set for the job, provide the full path here.
        /// </summary>
        public string ConfigFilePath
        {
            get { return _configFilePath; }
            set { _configFilePath = value; }
        }
        /// <summary>
        /// Object representing the parameters for the job
        /// </summary>
        public Object ParametersObject
        {
            get { return _parametersObject; }
            set { _parametersObject = value; }
        }
        /// <summary>
        /// Gets an instance of the parameters type in the default state
        /// </summary>
        /// <returns></returns>
        public object GetBlankParametersObject()
        {
            if (ParametersType == typeof(string)) return "";
            else return Activator.CreateInstance(ParametersType);
        }

        public object GetDeserialisedParametersObject(string serialisedParameterInformation)
        {
            //deserialise the parameter object
            //This code is also in SerialisationHelper, but I don't want this assembly to have
            //to reference any other NQueue assembly
            byte[] buffer = Encoding.UTF8.GetBytes(serialisedParameterInformation);
            MemoryStream stringAsStream = new MemoryStream(buffer, 0, buffer.Length, true, true);
            XmlSerializer serializer = new XmlSerializer(ParametersType);
            return serializer.Deserialize(stringAsStream);
        }
    }
}
