using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client.DataServices;

namespace NQueue.Client
{
    /// <summary>
    /// Handles cross process\server locking by using a shared key in the database
    /// </summary>
    /// <remarks>
    /// This functionality is used by the singleton flag on a job instance (wrapping
    /// the call in a mutex lock and marking instances that couldn't get the mutex as
    /// duplicates) and is available for use by client code (code that inherits from Job)
    /// </remarks>
    public class MutexManager : IDisposable
    {
        /// <summary>
        /// true if the class has cleanup work to do
        /// </summary>
        private bool _needsDisposing = false;
        /// <summary>
        /// true if the mutex has been obtained for this instance
        /// </summary>
        private bool _mutexObtained = false;
        /// <summary>
        /// Returns true if a mutex has been obtained for this instance
        /// </summary>
        public bool MutexObtained
        {
            get { return _mutexObtained; }
        }
        /// <summary>
        /// ID of the mutex, if obtained
        /// </summary>
        private int? _mutexID;
        /// <summary>
        /// Name of this server
        /// </summary>
        private readonly string _machineName = Environment.MachineName;
        /// <summary>
        /// Connection string - this is cached in the instance as the instance
        /// is serialised over to an app domain that may not have the connection string
        /// in its config file
        /// </summary>
        private string _connectionString;

        /// <summary>
        /// This constructor is called when passing the mutex manager into an executing
        /// client job.  The connection string is stored internally as it may not be in
        /// the config file for the client
        /// </summary>
        /// <param name="connectionString"></param>
        internal MutexManager(string connectionString)
        {
            this._connectionString = connectionString;
        }
        /// <summary>
        /// This constructor is called when wrapping instance executions in the mutex
        /// when the mutex manager is being used in the NQueue app domain.
        /// </summary>
        public MutexManager()
        {
            this._connectionString = DBCommon.ReadConnectionString(DBCommon.CONNECTIONSTRING_APP_KEY);
        }

        /// <summary>
        /// Attempts to obtain a mutex for the unique name given by mutex name
        /// </summary>
        /// <param name="mutexName">Name of the mutex</param>
        /// <returns>Returns true if the mutex get was successful, otherwise returns false</returns>
        public bool ObtainMutex(string mutexName, TimeSpan mutexTTL)
        {
            bool mutexObtained = DBMutex.ObtainMutex(this._connectionString, mutexName, _machineName, 
                mutexTTL, out _mutexID);
            if (mutexObtained)
            {
                this._needsDisposing = true;
                this._mutexObtained = true;
            }
            return mutexObtained;
        }


        public void Dispose()
        {
            if (this._needsDisposing)
            {
                ReleaseMutex();
                this._needsDisposing = false;
            }
        }
        /// <summary>
        /// Releases the mutex to be used by other processes
        /// </summary>
        public void ReleaseMutex()
        {
            if (this._mutexObtained)
            {
                try
                {
                    DBMutex.ReleaseMutex(this._connectionString, this._mutexID.Value, this._machineName);
                    this._mutexObtained = false;
                }
                catch (Exception exc)
                {
                    EventLogger.LogErrorEvent("Error attempting to release the mutex at ID " + this._mutexID.Value + " - this can cause jobs to fail as duplicate.  The ServerName field for this mutex ID should be set to NULL to rectify this situation", exc);
                    throw;
                }
            }
        }

        /// <summary>
        /// Deletes the mutex
        /// </summary>
        public void DeleteMutex()
        {
            if (this._mutexObtained)
            {
                try
                {
                    DBMutex.DeleteMutex(this._connectionString, this._mutexID.Value, this._machineName);
                    this._mutexObtained = false;
                }
                catch (Exception exc)
                {
                    EventLogger.LogErrorEvent("Error attempting to delete the mutex at ID " + this._mutexID.Value + " - this can cause jobs to fail as duplicate.  The row for the mutex ID should be deleted to rectify this situation", exc);
                    throw;
                }

            }
        }

    }
}
