using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;

namespace NQueue.DataServices
{
    public class DBCommon
    {

        /// <summary>
        /// Returns a connection object hooked up to the NQueue connection.
        /// The connection is not open.
        /// </summary>
        /// <param name="databaseKey">Key defining the database to connect to</param>
        /// <returns>A configured but closed connection</returns>
        public static SqlConnection GetConnection()
        {
            return GetConnection("cnn.NQueue");
        }

        /// <summary>
        /// Returns a connection object hooked up to the relevant connection.
        /// The connection is not open.
        /// </summary>
        /// <param name="databaseKey">Key defining the database to connect to</param>
        /// <returns>A configured but closed connection</returns>
        public static SqlConnection GetConnection(string databaseKey)
        {
            string connectionString = ReadConnectionString(databaseKey);
            if (connectionString == null || connectionString.Trim() == "")
                throw new InvalidOperationException("Could not find the connection string given by the key: " + databaseKey);
            //connection string found - return a new SQL connection configured with this string.
            return new SqlConnection(connectionString);
        }

        /// <summary>
        /// Gets a connection string given a database key
        /// </summary>
        /// <param name="databaseKey"></param>
        /// <returns></returns>
        public static string ReadConnectionString(string databaseKey)
        {
            ConnectionStringSettings connString = System.Configuration.ConfigurationManager.ConnectionStrings[databaseKey];
            if (connString != null)
            {
                return connString.ConnectionString;
            }
            else
            {
                string result = System.Configuration.ConfigurationManager.AppSettings[databaseKey];
                if (result == null) throw new InvalidOperationException("Could not find the connection string given by the key: " + databaseKey);
                return result;
            }
            
        }
        /// <summary>
        /// Gets a command object to a connection, set up to execute a stored proc
        /// </summary>
        /// <param name="cnn"></param>
        /// <param name="commandText"></param>
        /// <returns></returns>
        public static SqlCommand GetCommand(SqlConnection cnn, string commandText)
        {
            SqlCommand cmd = cnn.CreateCommand();
            cmd.CommandType = System.Data.CommandType.StoredProcedure;
            cmd.CommandText = commandText;
            return cmd;
        }

        /// <summary>
        /// Creates a return value parameter on a command object (type int) and returns it.
        /// </summary>
        /// <param name="cmd"></param>
        /// <returns></returns>
        public static SqlParameter ReturnValueParameter(SqlCommand cmd)
        {
            SqlParameter paramReturn = cmd.Parameters.Add("@RETURN_VALUE", System.Data.SqlDbType.Int);
            paramReturn.Direction = System.Data.ParameterDirection.ReturnValue;
            return paramReturn;
        }

        /// <summary>
        /// Returns DBNull.Value, but cast to an object type so that it can be used with the null coalescing operator - eg
        /// Paramter.Value = stringVariable_ ?? DBCommon.DBNull;
        /// Otherwise can't use ?? if operands are of non-related types
        /// </summary>
        public static object DBNull
        {
            get
            {
                return (object)System.DBNull.Value;
            }
        }

        /// <summary>
        /// Executes a parameterless stored procedure with no results
        /// </summary>
        /// <param name="cnn">Unopened connection pointing to the required database</param>
        /// <param name="storedProcName">Name of the stored procedure to execute</param>
        public static void ExecuteParameterlessNonQuery(SqlConnection cnn, string storedProcName)
        {
            using (cnn)
            {
                SqlCommand cmd = cnn.CreateCommand();
                cmd.CommandType = System.Data.CommandType.StoredProcedure;
                cmd.CommandText = storedProcName;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }
    }
}
