using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using NQueue.Entities;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for host instances
    /// </summary>
    public class DBHostInstance
    {
        /// <summary>
        /// Inserts a new host instance into the database
        /// </summary>
        /// <param name="hostID">Host ID for the host the instance is associated with</param>
        /// <param name="serverName">Servername of the server the instance runs on </param>
        /// <returns></returns>
        public static HostInstance InsertHostInstance(int hostID, string serverName)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInstanceInsert");
                cmd.Parameters.Add("@HostID", SqlDbType.Int).Value = hostID;
                cmd.Parameters.Add("@ServerName", SqlDbType.VarChar, 256).Value = serverName;
                SqlParameter paramReturn = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();
                HostInstance returnInstance = new HostInstance();
                returnInstance.HostID = hostID;
                returnInstance.ServerName = serverName;
                returnInstance.HostInstanceID = (int)paramReturn.Value;
                return returnInstance;
            }
        }
        /// <summary>
        /// Deletes a host instance from the database
        /// </summary>
        /// <param name="hostInstanceID"></param>
        public static void DeleteHostInstance(int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInstanceDelete");
                cmd.Parameters.Add("@HostInstanceID", SqlDbType.Int).Value = hostInstanceID;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Gets all host instances
        /// </summary>
        /// <returns></returns>
        public static List<HostInstance> GetAllHostInstances()
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInstancesSelectAll");
                return PopulateHostInstanceList(cnn, cmd);
            }
        }


        /// <summary>
        /// Gets all host instances for a given host
        /// </summary>
        /// <param name="hostID">Host ID to query</param>
        /// <returns></returns>
        public static List<HostInstance> GetAllHostInstances(int hostID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInstancesSelectAllForHost");
                cmd.Parameters.Add("@HostID", SqlDbType.Int).Value = hostID;
                return PopulateHostInstanceList(cnn, cmd);
            }
        }
        private static List<HostInstance> PopulateHostInstanceList(SqlConnection cnn, SqlCommand cmd)
        {
            cnn.Open();
            List<HostInstance> hostInstances = new List<HostInstance>();
            SqlDataReader results = cmd.ExecuteReader();
            while (results.Read())
            {
                HostInstance instance = new HostInstance();
                instance.HostInstanceID = results.GetInt32(0);
                instance.HostID = results.GetInt32(1);
                instance.ServerName = results.GetString(2);
                instance.HostName = results.GetString(3);
                hostInstances.Add(instance);
            }
            return hostInstances;
        }

        /// <summary>
        /// Gets a specific host instance
        /// </summary>
        /// <param name="hostInstanceID">Database Id of the required host instance</param>
        /// <returns></returns>
        public static HostInstance GetHostInstance(int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInstancesSelectOne");
                cmd.Parameters.Add("@HostInstanceID", SqlDbType.Int).Value = hostInstanceID;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("hostInstanceID", "HostInstance not found for host instance ID " + hostInstanceID.ToString());
                }
                else
                {
                    HostInstance targetInstance = new HostInstance();
                    targetInstance.HostID = results.GetInt32(0);
                    targetInstance.ServerName = results.GetString(1);
                    targetInstance.HostName = results.GetString(2);
                    targetInstance.HostInstanceID = hostInstanceID;
                    return targetInstance;
                }
            }
        }
    }
}
