using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using NQueue.Entities;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for job definitions
    /// </summary>
    public class DBJobDefinition
    {
        /// <summary>
        /// Inserts a new job definition into the database
        /// </summary>
        /// <param name="jobName"></param>
        /// <param name="jobDescription"></param>
        /// <param name="jobAssemblyStrongName"></param>
        /// <param name="jobClassType"></param>
        /// <param name="jobConfigFile"></param>
        /// <returns>Job definition instance representing the newly inserted job</returns>
        public static JobDefinition InsertJobDefinition(string jobName, string jobDescription, string jobAssemblyStrongName, string jobClassType, string jobConfigFile)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobDefinitionInsert");
                cmd.Parameters.Add("@JobName", SqlDbType.VarChar, 70).Value = jobName;
                cmd.Parameters.Add("@JobDescription", SqlDbType.VarChar, 300).Value = jobDescription;
                cmd.Parameters.Add("@JobAssemblyStrongName", SqlDbType.NVarChar, 256).Value = jobAssemblyStrongName;
                cmd.Parameters.Add("@JobClassType", SqlDbType.NVarChar, 256).Value = jobClassType;
                cmd.Parameters.Add("@JobConfigFile", SqlDbType.NVarChar, -1).Value = jobConfigFile;
                SqlParameter paramReturn = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();
                JobDefinition returnJob = new JobDefinition();
                returnJob.JobName = jobName;
                returnJob.JobDescription = jobDescription;
                returnJob.JobAssemblyStrongName = jobAssemblyStrongName;
                returnJob.JobClassType = jobClassType;
                returnJob.JobConfigFile = jobConfigFile;
                returnJob.JobDefinitionID = (int)paramReturn.Value;
                return returnJob;
            }
        }

        /// <summary>
        /// Deletes a job definition from the database
        /// </summary>
        /// <param name="jobDefinitionID"></param>
        public static void DeleteJobDefinition(int jobDefinitionID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobDefinitionDelete");
                cmd.Parameters.Add("@JobDefinitionID", SqlDbType.Int).Value = jobDefinitionID;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }
        /// <summary>
        /// Returns all job definitions in the database
        /// </summary>
        /// <returns></returns>
        public static List<JobDefinition> GetAllJobDefinitions()
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobDefinitionSelectAll");
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                List<JobDefinition> allJobs = new List<JobDefinition>();
                while (results.Read())
                {
                    JobDefinition newJob = PopulateJobDefinitionFromReader(results);
                    allJobs.Add(newJob);
                }
                return allJobs;
            }
        }

        private static JobDefinition PopulateJobDefinitionFromReader(SqlDataReader results)
        {
            JobDefinition newJob = new JobDefinition();
            newJob.JobDefinitionID = results.GetInt32(0);
            newJob.JobName = results.GetString(1);
            newJob.JobDescription = results.GetString(2);
            newJob.JobAssemblyStrongName = results.GetString(3);
            newJob.JobClassType = results.GetString(4);
            newJob.JobConfigFile = results.GetString(5);
            return newJob;
        }

        /// <summary>
        /// Gets a job definition from the database
        /// </summary>
        /// <param name="jobDefinitionID">ID of the job def in the database</param>
        /// <returns></returns>
        public static JobDefinition GetJobDefinition(int jobDefinitionID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobDefinitionsSelectOne");
                cmd.Parameters.Add("@JobDefinitionID", SqlDbType.Int).Value = jobDefinitionID;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("jobDefinitionID", "Job Definition not found for Job Definition ID " + jobDefinitionID.ToString());
                }
                else
                {
                    return PopulateJobDefinitionFromReader(results);
                }
            }
        }
    }
}
