using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using NQueue.Entities;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for Job Instances
    /// </summary>
    public class DBJobInstance
    {
        /// <summary>
        /// Inserts a new job instance into the database
        /// </summary>
        /// <param name="jobDefinitionID"></param>
        /// <param name="jobInstanceName"></param>
        /// <param name="hostID"></param>
        /// <param name="instanceTTLTimeSpan"></param>
        /// <param name="jobParameters"></param>
        /// <returns>Returns a populated job instance object including the job instance ID (job name is not set)</returns>
        public static JobInstance InsertJobInstance(int jobDefinitionID, string jobInstanceName, int hostID, TimeSpan instanceTTLTimeSpan, string jobParameters, bool isSingleton)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceInsert");
                cmd.Parameters.Add("@JobDefinitionID", SqlDbType.Int).Value = jobDefinitionID;
                cmd.Parameters.Add("@JobInstanceName", SqlDbType.VarChar, 70).Value = jobInstanceName;
                cmd.Parameters.Add("@HostID", SqlDbType.Int).Value = hostID;
                //@InstanceTTLTimeSpan should be set to the number of minutes that an executing job should be allowed to run before being terminated
                cmd.Parameters.Add("@InstanceTTLTimeSpan", SqlDbType.Int).Value = (int) instanceTTLTimeSpan.TotalMinutes;
                cmd.Parameters.Add("@JobParameters", SqlDbType.NVarChar, -1).Value = jobParameters;
                cmd.Parameters.Add("@Singleton", SqlDbType.Bit).Value = isSingleton;
                SqlParameter paramReturn = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();

                JobInstance returnInstance = new JobInstance();
                returnInstance.JobDefinitionID = jobDefinitionID;
                returnInstance.JobInstanceName = jobInstanceName;
                returnInstance.HostID = hostID;
                returnInstance.InstanceTTLTimeSpan = instanceTTLTimeSpan;
                returnInstance.JobParameters = jobParameters;
                returnInstance.JobInstanceID = (int)paramReturn.Value;
                returnInstance.JobInstanceState = JobInstanceStates.Enabled;
                returnInstance.IsSingleton = isSingleton;

                return returnInstance;
            }
        }

        /// <summary>
        /// Deletes a job instance from the database
        /// </summary>
        /// <param name="jobInstanceID">Instance ID of the job</param>
        public static void DeleteJobInstance(int jobInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceDelete");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Returns a specific job instance from the database
        /// </summary>
        /// <param name="jobInstanceID"></param>
        /// <returns></returns>
        public static JobInstance GetJobInstance(int jobInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSelectOne");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("jobInstanceID", "Job instance not found for Job Instance ID " + jobInstanceID.ToString());
                }
                else
                {
                    return PopulateJobInstanceFromReader(results);
                }
            }
        }

        /// <summary>
        /// Returns a specific job instance from the database
        /// </summary>
        /// <param name="jobInstanceID"></param>
        /// <returns></returns>
        public static JobInstance GetJobInstance(string jobInstanceName)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_GetJobInstanceByName");
                cmd.Parameters.Add("@JobInstanceName", SqlDbType.VarChar, 70).Value = jobInstanceName;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("jobInstanceName", "Job instance not found for Job Instance name " + jobInstanceName);
                }
                else
                {
                    JobInstance returnInstance = PopulateJobInstanceFromReader(results);
                    if (results.Read())
                    {
                        //shouldn't be more than one record returned
                        throw new InvalidOperationException("More than one job instance returned for job instance with name: " + jobInstanceName);
                    }
                    return returnInstance;
                }
            }
        }

        /// <summary>
        /// Returns all job instances from the database
        /// </summary>
        /// <returns></returns>
        public static List<JobInstance> GetAllJobInstances()
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSelectAll");
                return GetJobInstanceListFromCommandObject(cnn, cmd);
            }
        }

        /// <summary>
        /// Returns all job instances from the database for a given job
        /// </summary>
        /// <param name="jobDefinitionID">Job definition ID to search for instances for</param>
        /// <returns></returns>
        public static List<JobInstance> GetAllJobInstances(int jobDefinitionID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSelectAllForJob");
                cmd.Parameters.Add("@JobDefinitionID", SqlDbType.Int).Value = jobDefinitionID;
                return GetJobInstanceListFromCommandObject(cnn, cmd);
            }
        }

        private static List<JobInstance> GetJobInstanceListFromCommandObject(SqlConnection cnn, SqlCommand cmd)
        {
            cnn.Open();
            SqlDataReader results = cmd.ExecuteReader();
            List<JobInstance> allInstances = new List<JobInstance>();
            while (results.Read())
            {
                allInstances.Add(PopulateJobInstanceFromReader(results));
            }
            return allInstances;
        }

        private static JobInstance PopulateJobInstanceFromReader(SqlDataReader results)
        {
            JobInstance newInstance = new JobInstance();
            newInstance.JobInstanceID = results.GetInt32(0);
            newInstance.JobDefinitionID = results.GetInt32(1);
            newInstance.JobInstanceName = results.GetString(2);
            newInstance.HostID = results.GetInt32(3);
            newInstance.InstanceTTLTimeSpan = new TimeSpan(0, results.GetInt32(4), 0);
            newInstance.JobParameters = results.GetString(5);
            newInstance.JobName = results.GetString(6);
            newInstance.JobInstanceState = JobInstance.ConvertStateIDToState(results.GetInt32(7));
            newInstance.IsSingleton = results.GetBoolean(8);
            return newInstance;
        }

        /// <summary>
        /// Inserts a schedule for a job instance into the database
        /// </summary>
        /// <param name="jobInstanceID">Job instance being scheduled - a job instance can have more that one schedule</param>
        /// <param name="ScheduleTypeID">Type ID of the schedule type - daily, monthly, etc</param>
        /// <param name="scheduleParameters">Serialised XML of an instance of the relevant parameters object for the schedule type - to describe eg "the 3rd day of the week"</param>
        /// <returns>Returns the newly inserted job instance schedule ID</returns>
        public static int InsertInstanceSchedule(int jobInstanceID, int scheduleTypeID, string scheduleParameters)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstance_InsertSchedule");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
                cmd.Parameters.Add("@ScheduleParameters", SqlDbType.NVarChar, -1).Value = scheduleParameters;
                SqlParameter returnParam = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();
                return (int)returnParam.Value;
            }
        }
        /// <summary>
        /// Deletes a job instance schedule item
        /// </summary>
        /// <param name="jobInstanceScheduleID"></param>
        public static void DeleteInstanceSchedule(int jobInstanceScheduleID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstance_DeleteSchedule");
                cmd.Parameters.Add("@JobInstanceScheduleID", SqlDbType.Int).Value = jobInstanceScheduleID;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Returns all job instance schedules for a job instance
        /// </summary>
        /// <param name="jobInstanceID"></param>
        /// <returns></returns>
        public static List<JobInstanceSchedule> GetJobInstanceSchedulesForJobInstance(int jobInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSchedule_GetForJob");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                cnn.Open();
                return PopulateJobInstanceScheduleList(cmd);
            }
        }


        /// <summary>
        /// Marks a job instance schedule as having been processed
        /// </summary>
        /// <param name="cnn">SQL Connection on which to run the update - connection must be open and is not closed afterwards</param>
        /// <param name="jobInstanceScheduleID"></param>
        public static void UpdateInstanceScheduleToShowProcessed(int jobInstanceScheduleID, SqlConnection cnn, SqlTransaction tran)
        {
            if (cnn.State != ConnectionState.Open) throw new InvalidOperationException("Connection passed in was not open, was: " + cnn.State.ToString());
            SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSchedule_MarkLastProcessed");
            cmd.Transaction = tran;
            cmd.Parameters.Add("@JobInstanceScheduleID", SqlDbType.Int).Value = jobInstanceScheduleID;
            cmd.ExecuteNonQuery();
        }


        /// <summary>
        /// Returns all job instance schedules for a schedule type
        /// </summary>
        /// <param name="jobInstanceID"></param>
        /// <returns></returns>
        public static List<JobInstanceSchedule> GetJobInstanceSchedulesForScheduleType(int scheduleTypeID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceSchedule_GetForScheduleType");
                cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
                cnn.Open();
                return PopulateJobInstanceScheduleList(cmd);
            }
        }

        private static List<JobInstanceSchedule> PopulateJobInstanceScheduleList(SqlCommand cmd)
        {
            SqlDataReader results = cmd.ExecuteReader();
            List<JobInstanceSchedule> schedules = new List<JobInstanceSchedule>();
            while (results.Read())
            {
                JobInstanceSchedule newSchedule = new JobInstanceSchedule();
                newSchedule.JobInstanceScheduleID = results.GetInt32(0);
                newSchedule.JobInstanceID = results.GetInt32(1);
                newSchedule.ScheduleTypeID = results.GetInt32(2);
                if (results.IsDBNull(3)) newSchedule.InstanceScheduleLastRun = null; else newSchedule.InstanceScheduleLastRun = results.GetDateTime(3);
                newSchedule.ScheduleParameters = results.GetString(4);
                newSchedule.ScheduleTypeName = results.GetString(5);
                schedules.Add(newSchedule);
            }
            return schedules;
        }

        /// <summary>
        /// Updates a job instance to a different state (e.g. disabled)
        /// </summary>
        /// <param name="jobInstanceID">ID of the job instance to update</param>
        /// <param name="jobInstanceState">New state for the job instance</param>
        public static void UpdateJobInstanceState(int jobInstanceID, JobInstanceStates jobInstanceState)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobInstanceUpdateState");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                cmd.Parameters.Add("@JobInstanceStateID", SqlDbType.Int).Value = (int)jobInstanceState;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }
    }
}
