using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using NQueue.Entities;
using NQueue.Client;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for scheduling activities
    /// </summary>
    public class DBScheduling
    {
        /// <summary>
        /// Gets a list of all schedules currently in the database
        /// </summary>
        /// <returns></returns>
        public static List<ScheduleType> GetAllScheduleTypes()
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobSchedule_GetAllSchedules");
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                List<ScheduleType> schedules = new List<ScheduleType>();
                while (results.Read())
                {
                    ScheduleType foundScheduleType = PopulateScheduleTypeRecord(results);
                    schedules.Add(foundScheduleType);
                }
                return schedules;
            }
        }

        private static ScheduleType PopulateScheduleTypeRecord(SqlDataReader results)
        {
            ScheduleType foundScheduleType = new ScheduleType();
            foundScheduleType.ScheduleTypeID = results.GetInt32(0);
            foundScheduleType.ScheduleTypeName = results.GetString(1);
            foundScheduleType.CurrentlyProcessing = results.GetBoolean(2);
            if (!results.IsDBNull(3)) foundScheduleType.LastProcessed = results.GetDateTime(3); else foundScheduleType.LastProcessed = null;
            if (!results.IsDBNull(4)) foundScheduleType.NextProcessDue = results.GetDateTime(4); else foundScheduleType.NextProcessDue = null;
            foundScheduleType.ProcessingHostInstanceID = results.IsDBNull(5) ? null : (int?)results.GetInt32(5);
            foundScheduleType.ScheduleTypeStrongName = results.GetString(6);
            foundScheduleType.ScheduleTypeClassName = results.GetString(7);
            return foundScheduleType;
        }
        /// <summary>
        /// Marks a schedule type as in process
        /// </summary>
        /// <param name="scheduleTypeID">Schedule type to mark as being processed</param>
        /// <param name="lastProcessingDate">Processing date the caller thinks the schedule was last processed</param>
        /// <param name="hostInstanceID">Host instance ID of the caller</param>
        /// <returns>Returns true if the update was successful, returns false if a concurrency violation meant the update could not occur</returns>
        public static bool MarkScheduleTypeAsBeingProcessed(int scheduleTypeID, DateTime? lastProcessingDate, int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobSchedule_MarkScheduleTypeAsBeingProcessed");
                cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
                cmd.Parameters.Add("@LastProcessedDate", SqlDbType.DateTime).Value = lastProcessingDate ?? DBCommon.DBNull;
                cmd.Parameters.Add("@ProcessingHostInstanceID", SqlDbType.Int).Value = hostInstanceID;
                cnn.Open();
                try
                {
                    cmd.ExecuteNonQuery();
                    return true;
                }
                catch (SqlException exc)
                {
                    if (exc.Class == 16 && exc.Message == "Concurrency violation")
                    {
                        //we expect to get this when more than one host instance wants to process a schedule type
                        //so just swallow this error but indicate to the caller that we couldn't take ownership of
                        //the schedule type for this run
                        return false;
                    }
                    else
                    {
                        //otherwise its some other sort of SQL error - log and chuck it
                        EventLogger.LogErrorEvent("Error trying to mark schedule type as in process", exc);
                        throw;
                    }
                }
            }
        }
        /// <summary>
        /// Marks a schedule type as having been processed
        /// </summary>
        /// <param name="scheduleTypeID">Schedule type to mark as being processed</param>
        /// <param name="lastProcessingDate">Processing date the caller thinks the schedule was last processed</param>
        /// <param name="hostInstanceID">Host instance ID of the caller</param>
        /// <param name="nextProcessingDate">Next date the schedule type is due to be processed</param>
        public static void MarkScheduleTypeProcessingComplete(int scheduleTypeID, DateTime? lastProcessingDate, DateTime nextProcessingDate, int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                cnn.Open();
                MarkScheduleTypeProcessingComplete(scheduleTypeID, lastProcessingDate, nextProcessingDate, hostInstanceID, cnn, null);
            }
        }

        /// <summary>
        /// Marks a schedule type as having been processed
        /// </summary>
        /// <param name="scheduleTypeID">Schedule type to mark as being processed</param>
        /// <param name="lastProcessingDate">Processing date the caller thinks the schedule was last processed</param>
        /// <param name="hostInstanceID">Host instance ID of the caller</param>
        /// <param name="nextProcessingDate">Next date the schedule type is due to be processed</param>
        public static void MarkScheduleTypeProcessingComplete(int scheduleTypeID, DateTime? lastProcessingDate, DateTime nextProcessingDate, int hostInstanceID, SqlConnection cnn, SqlTransaction tran)
        {
            SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobSchedule_ShowScheduleProcessed");
            if (tran != null) cmd.Transaction = tran;
            cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
            cmd.Parameters.Add("@LastProcessedDate", SqlDbType.DateTime).Value = lastProcessingDate ?? DBCommon.DBNull;
            cmd.Parameters.Add("@NextProcessingDate", SqlDbType.DateTime).Value = nextProcessingDate;
            cmd.Parameters.Add("@ProcessingHostInstanceID", SqlDbType.Int).Value = hostInstanceID;
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (SqlException exc)
            {

                EventLogger.LogErrorEvent("Error marking schedule processing as complete - schedule type table may be in an inconsistent state", exc);
                throw;      //shouldn't get this error, so need to rethrow it
            }
        }

        /// <summary>
        /// Removes the "mark" from a schedule type showing its being processed
        /// by a given host instance.  The mark should be removed if processing is
        /// unsuccessful.
        /// </summary>
        /// <param name="scheduleTypeID"></param>
        /// <param name="hostInstanceID"></param>
        public static void RemoveScheduleTypeProcessingMark(int scheduleTypeID, int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobSchedule_RemoveProcessMark");
                cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
                cmd.Parameters.Add("@ProcessingHostInstanceID", SqlDbType.Int).Value = hostInstanceID;
                cmd.ExecuteNonQuery();
            }
        }

        public static ScheduleType GetSchedule(int scheduleTypeID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_JobSchedule_GetScheduleType");
                cmd.Parameters.Add("@ScheduleTypeID", SqlDbType.Int).Value = scheduleTypeID;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("scheduleTypeID", "Schedule Type not found for Schedule Type ID " + scheduleTypeID.ToString());
                }
                else
                {
                    return PopulateScheduleTypeRecord(results);
                }

            }
        }
    }
}
