using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using NQueue.Entities;
using NQueue.Client;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for work queue activities
    /// </summary>
    public class DBWorkQueue
    {

        /// <summary>
        /// Puts a work item on the queue.
        /// A work item is a job instance ready to be executed on the queue.  The job instance
        /// is associated with a host and work items are dequeued based on this association with the host
        /// </summary>
        /// <param name="jobInstanceID">Job instance to enqueue</param>
        /// <param name="dateToExecute">Date before which the work item should not be dequeued (and executed)</param>
        public static long EnqueueWorkItem(int jobInstanceID, string serverAffinity, DateTime dateToExecute)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                cnn.Open();
                return EnqueueWorkItem(jobInstanceID, serverAffinity, dateToExecute, cnn, null);
            }
        }

        /// <summary>
        /// Puts a work item on the queue.
        /// A work item is a job instance ready to be executed on the queue.  The job instance
        /// is associated with a host and work items are dequeued based on this association with the host
        /// </summary>
        /// <param name="jobInstanceID">Job instance to enqueue</param>
        /// <param name="dateToExecute">Date before which the work item should not be dequeued (and executed)</param>
        public static long EnqueueWorkItem(int jobInstanceID, DateTime dateToExecute)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                cnn.Open();
                return EnqueueWorkItem(jobInstanceID, null, dateToExecute, cnn, null);
            }
        }

        /// <summary>
        /// Puts a work item on the queue for immediate execution.
        /// A work item is a job instance ready to be executed on the queue.  The job instance
        /// is associated with a host and work items are dequeued based on this association with the host
        /// </summary>
        /// <param name="jobInstanceID">Job instance to enqueue</param>
        /// <param name="dateToExecute">Date before which the work item should not be dequeued (and executed)</param>
        /// <param name="cnn">Connection object to use - should already be open</param>
        /// <param name="tran">SQL transaction to apply to the update - pass in null if no transaction required</param>
        private static long EnqueueWorkItem(WorkItemToEnqueue workItem, SqlConnection cnn, SqlTransaction tran)
        {
            return EnqueueWorkItem(workItem.JobInstanceID, workItem.ServerAffinity, workItem.DateToExecute, cnn, tran);
        }



        /// <summary>
        /// Puts a work item on the queue.
        /// A work item is a job instance ready to be executed on the queue.  The job instance
        /// is associated with a host and work items are dequeued based on this association with the host
        /// </summary>
        /// <param name="jobInstanceID">Job instance to enqueue</param>
        /// <param name="dateToExecute">Date before which the work item should not be dequeued (and executed)</param>
        /// <param name="cnn">Connection object to use - should already be open</param>
        /// <param name="tran">SQL transaction to apply to the update - pass in null if no transaction required</param>
        private static long EnqueueWorkItem(int jobInstanceID, string serverAffinity, DateTime dateToExecute, SqlConnection cnn, SqlTransaction tran)
        {
            SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_WorkQueue_EnqueueWorkItem");
            if (tran != null) cmd.Transaction = tran;
            cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
            cmd.Parameters.Add("@DateToExecute", SqlDbType.DateTime).Value = dateToExecute;
            cmd.Parameters.Add("@ServerAffinity", SqlDbType.VarChar, 256).Value = serverAffinity ?? DBCommon.DBNull;
            SqlParameter paramReturn = cmd.Parameters.Add("@RETURN_VALUE", System.Data.SqlDbType.BigInt);
            paramReturn.Direction = System.Data.ParameterDirection.ReturnValue;
            cmd.ExecuteNonQuery();
            return Convert.ToInt64(paramReturn.Value);
        }

        /// <summary>
        /// Dequeues and returns a work item from the work queue.
        /// Returns null if there are no work items to execute
        /// </summary>
        /// <param name="hostInstanceID">Host instance ID of the process dequeuing work</param>
        /// <returns>Returns a just-dequeued work item or null if there is no work to do</returns>
        public static DequeuedWorkItem DequeueWorkItem(int hostInstanceID)
        {
            return DequeueWorkItem(hostInstanceID, System.Environment.MachineName);
        }

        /// <summary>
        /// Dequeues and returns a work item from the work queue.
        /// Returns null if there are no work items to execute
        /// </summary>
        /// <param name="hostInstanceID">Host instance ID of the process dequeuing work</param>
        /// <param name="dequeuingServerName">Name of the server this procedure is being called from</param>
        /// <returns>Returns a just-dequeued work item or null if there is no work to do</returns>
        public static DequeuedWorkItem DequeueWorkItem(int hostInstanceID, string dequeuingServerName)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_WorkQueue_DequeueWorkItem");
                cmd.Parameters.Add("@HostInstanceID", SqlDbType.Int).Value = hostInstanceID;
                cmd.Parameters.Add("@ServerName", SqlDbType.VarChar, 256).Value = dequeuingServerName ?? DBCommon.DBNull;
                cnn.Open();
                SqlDataReader result = cmd.ExecuteReader();
                if (!result.Read())
                {
                    //return null if no work to do
                    return null;
                }
                DequeuedWorkItem workItem = new DequeuedWorkItem();
                workItem.WorkItemID = result.GetInt64(0);
                workItem.JobInstanceID = result.GetInt32(1);
                workItem.JobDefinitionID = result.GetInt32(2);
                workItem.JobInstanceName = result.GetString(3);
                workItem.JobParameters = result.GetString(4);
                workItem.JobName = result.GetString(5);
                workItem.JobAssemblyStrongName = result.GetString(6);
                workItem.JobClassType = result.GetString(7);
                workItem.JobConfigFile = result.GetString(8);
                workItem.JobInstanceState = JobInstance.ConvertStateIDToState(result.GetInt32(9));
                workItem.IsSingleton = result.GetBoolean(10);
                workItem.InstanceTTLTimeSpan = new TimeSpan(0, result.GetInt32(11), 0);
                return workItem;
            }
        }

        /// <summary>
        /// Marks a work item as complete on the queue
        /// </summary>
        /// <param name="workItemID"></param>
        /// <param name="result"></param>
        public static void MarkWorkItemComplete(long workItemID, ExecutionResults result)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_WorkQueue_MarkItemCompleted");
                cmd.Parameters.Add("@WorkItemID", SqlDbType.BigInt).Value = workItemID;
                cmd.Parameters.Add("@WorkItemStatusID", SqlDbType.Int).Value = Convert.ToInt32(result);
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }

        public static void TransactionallyEnqueueWorkScheduleListAndMarkComplete(List<WorkItemToEnqueue> allWorkItemsToEnqueue, List<JobInstanceSchedule> instanceSchedules, int scheduleTypeID, DateTime? lastProcessedScheduleType, DateTime estimatedNextScheduleTypeProcessingDate, int hostInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                cnn.Open();
                SqlTransaction tran = cnn.BeginTransaction();
                try
                {
                    //insert all work items
                    foreach (WorkItemToEnqueue workItem in allWorkItemsToEnqueue)
                    {
                        EnqueueWorkItem(workItem, cnn, tran);
                    }
                    //update each JobInstanceSchedule to show it has been processed
                    foreach (JobInstanceSchedule schedule in instanceSchedules)
                    {
                        if (schedule.JobInstanceScheduleWasJustProcessed)
                            DBJobInstance.UpdateInstanceScheduleToShowProcessed(schedule.JobInstanceScheduleID, cnn, tran);
                    }
                    //mark the schedule type as processed
                    DBScheduling.MarkScheduleTypeProcessingComplete(scheduleTypeID, lastProcessedScheduleType, estimatedNextScheduleTypeProcessingDate, hostInstanceID, cnn, tran);

                    //commit the transaction to the database
                    tran.Commit();
                }
                catch (Exception exc)
                {
                    tran.Rollback();
                    EventLogger.LogErrorEvent(string.Format("Failed to enqueue work items.  Tried to enqueue {0} items, for schedule type id {1}", allWorkItemsToEnqueue.Count, scheduleTypeID), exc);
                    throw;
                }
            }
        }
    }
}
