using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using NQueue.BusinessServices.Scheduling;
using NQueue.BusinessServices.Jobs;
using NQueue.BusinessServices.System;
using NQueue.Client;

namespace NQueue.Service.Server
{
    /// <summary>
    /// This class implements the NQueue server.
    /// Instantiate this and hold a reference to it to run the service
    /// </summary>
    public class ServiceImplementation : IDisposable
    {
        private int _hostInstanceID;
        private Timer _schedulingTimer;
        private Timer _dequeuingTimer;
        private bool _timersNeedDisposing = false;
        private bool _allowImpersonation = false;

        /// <summary>
        /// ctor
        /// </summary>
        /// <param name="hostInstanceID">The ID of the host instance this service represents</param>
        /// <param name="allowImpersonation">If set to true, this host instance will be allowed to run on an incorrect server.  This should only be set to true when running the test service in development as a convenience to the developer</param>
        public ServiceImplementation(int hostInstanceID, bool allowImpersonation)
        {
            this._hostInstanceID = hostInstanceID;
            this._allowImpersonation = allowImpersonation;
        }
        /// <summary>
        /// Starts the service running - sets the scheduler and dequeuer running
        /// on a periodic basis
        /// </summary>
        public void StartService()
        {
            //Check if this server is allowed to run as this host instance
            if (!SystemManager.CanServerRunHostInstance(_hostInstanceID))
            {
                if (this._allowImpersonation)
                {
                    EventLogger.LogInformationEvent("Allowing service to run as impersonation was set to true.  If this is a production machine this is incorrect and should be changed");
                }
                else
                {
                    throw new InvalidOperationException("Server is not authorised to run this host instance");
                }
            }
            _schedulingTimer = new Timer(new TimerCallback(OnSchedule), null, new TimeSpan(0, 0, 0), new TimeSpan(0, 1, 0));
            _dequeuingTimer = new Timer(new TimerCallback(OnDequeue), null, new TimeSpan(0, 0, 5), new TimeSpan(0, 0, 5));
            _timersNeedDisposing = true;
        }
        /// <summary>
        /// Stops the service - the scheduler and dequeuer are stopped
        /// </summary>
        public void StopService()
        {
            DisposeTimers();
        }
        /// <summary>
        /// Idempotently disposes the scheduler and dequeuer timers
        /// </summary>
        private void DisposeTimers()
        {
            if (_timersNeedDisposing)
            {
                WaitHandle scheduleWait = new ManualResetEvent(false);
                WaitHandle dequeueWait = new ManualResetEvent(false);
                _schedulingTimer.Dispose(scheduleWait);
                _dequeuingTimer.Dispose(dequeueWait);
                _timersNeedDisposing = false;
                scheduleWait.WaitOne();
                dequeueWait.WaitOne();
            }
        }

        public void Dispose()
        {
            DisposeTimers();
        }

        private void OnSchedule(object state)
        {
            try
            {
                Scheduler.CheckAndExecuteSchedules(_hostInstanceID);
            }
            catch (Exception exc)
            {
                EventLogger.LogErrorEvent("Error processing schedule check", exc);
            }
        }

        private void OnDequeue(object state)
        {
            try
            {
                JobDequeuer.Execute(_hostInstanceID);
            }
            catch (Exception exc)
            {
                EventLogger.LogErrorEvent("Error processing work dequeue operation", exc);
            }
        }

    }
}
