﻿using NUnit.Framework;
using System;
using System.Text;
using System.Collections.Generic;
using NQueue.DataServices;
using NQueue.Entities;
namespace NQueue.UnitTests.DataServices
{
    /// <summary>
    ///This is a test class for NQueue.DataServices.DBScheduling and is intended
    ///to contain all NQueue.DataServices.DBScheduling Unit Tests
    ///</summary>
    [TestFixture]
    public class DBSchedulingTest
    {


        private Host _testHost;
        private HostInstance _testHostInstance;
        private JobDefinition _testJobDef;
        private JobInstance _testJobInstance;

        [TestFixtureSetUp]
        public void MyClassInitialize()
        {
            string jobParams = string.Format("<ParameteredJobTypeParameters><Param1>{0}</Param1><Param2>{1}</Param2></ParameteredJobTypeParameters>", 2, 2);
            string jobAssemblyStrongName = "NQueue.UnitTests, Version=1.0.0.0, Culture=neutral, PublicKeyToken=d5f1da7c501d0b59";
            string jobClassType = "NQueue.UnitTests.ParameteredJobType";
            string jobConfigFile = @"TestBatchJobConfiguration.dll.config";


            _testHost = DBHost.InsertHost("TestHost " + Guid.NewGuid().ToString());
            _testHostInstance = DBHostInstance.InsertHostInstance(_testHost.HostID, "Server " + Guid.NewGuid().ToString());
            _testJobDef = DBJobDefinition.InsertJobDefinition("Job def " + Guid.NewGuid().ToString(), "JobDesc", jobAssemblyStrongName, jobClassType, jobConfigFile);
            _testJobInstance = DBJobInstance.InsertJobInstance(_testJobDef.JobDefinitionID, "Job inst" + Guid.NewGuid().ToString(), _testHost.HostID, new TimeSpan(1, 15, 0), jobParams, false);
        }


        [TestFixtureTearDown]
        public void MyClassCleanup()
        {
            DBJobInstance.DeleteJobInstance(_testJobInstance.JobInstanceID);
            DBJobDefinition.DeleteJobDefinition(_testJobDef.JobDefinitionID);
            DBHostInstance.DeleteHostInstance(_testHostInstance.HostInstanceID);
            DBHost.DeleteHost(_testHost.HostID);
        }

        /// <summary>
        /// A test for MarkScheduleTypeAsBeingProcessed (int, DateTime, int)
        ///</summary>
        [Test]
        public void ScheduleTypeProcessingTest()
        {
            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            Assert.IsTrue(schedules.Count > 0);
            //work with the first schedule item (these are seed data)
            ScheduleType schedule = schedules[0];
            Assert.IsNull(schedule.ProcessingHostInstanceID);
            //Mark it as being processed
            Assert.IsTrue(DBScheduling.MarkScheduleTypeAsBeingProcessed(schedule.ScheduleTypeID, schedule.LastProcessed, _testHostInstance.HostInstanceID));
            //Mark processing as complete
            DBScheduling.MarkScheduleTypeProcessingComplete(schedule.ScheduleTypeID, schedule.LastProcessed, DateTime.Now, _testHostInstance.HostInstanceID);
        }

        [Test]
        public void ScheduleTypeProcessingTest_ConcurrencyViolation()
        {
            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            Assert.IsTrue(schedules.Count > 0);
            //work with the first schedule item (these are seed data)
            ScheduleType schedule = schedules[0];
            Assert.IsNull(schedule.ProcessingHostInstanceID);
            //manually go into the database and tweak the last processed date on the schedule id we want to use -
            //this should cause a concurrency violation when we try and mark it as being processed which should 
            //cause MarkScheduleTypeAsBeingProcessed to return false
            string sqlTweak = string.Format("update ScheduleTypes set LastProcessed = '{0}' where ScheduleTypeID = {1}", DateTime.Now.AddSeconds(1).ToString("yyyy-MM-dd hh:mm:ss"), schedule.ScheduleTypeID);
            Common.ExecuteNonQuery(sqlTweak);
            //Mark it as being processed - this should fail and return false
            Assert.IsFalse(DBScheduling.MarkScheduleTypeAsBeingProcessed(schedule.ScheduleTypeID, schedule.LastProcessed, _testHostInstance.HostInstanceID));
        }

        [Test]
        public void ScheduleAndEnqueueProcessingTest()
        {
            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            Assert.IsTrue(schedules.Count > 0);

            //work with the first schedule item (these are seed data)
            ScheduleType schedule = schedules[0];
            Assert.IsNull(schedule.ProcessingHostInstanceID);
            //Mark it as being processed
            Assert.IsTrue(DBScheduling.MarkScheduleTypeAsBeingProcessed(schedule.ScheduleTypeID, schedule.LastProcessed, _testHostInstance.HostInstanceID));
            //add some job instance schedules for the type
            int instanceScheduleID1 = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, "PARAMS");
            int instanceScheduleID2 = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, "PARAMS");
            int instanceScheduleID3 = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, "PARAMS");
            int instanceScheduleID4 = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, "PARAMS");
            int instanceScheduleID5 = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, "PARAMS");
            
            try
            {
                //get the just created schedule instances
                List<JobInstanceSchedule> scheduleInstances = DBJobInstance.GetJobInstanceSchedulesForJobInstance(_testJobInstance.JobInstanceID);
                //get some work items to enqueue
                List<WorkItemToEnqueue> itemsToEnqueue = new List<WorkItemToEnqueue>();
                itemsToEnqueue.Add(new WorkItemToEnqueue(_testJobInstance.JobInstanceID, System.DateTime.Now.AddMinutes(-1)));
                itemsToEnqueue.Add(new WorkItemToEnqueue(_testJobInstance.JobInstanceID, System.DateTime.Now.AddMinutes(-2)));
                itemsToEnqueue.Add(new WorkItemToEnqueue(_testJobInstance.JobInstanceID, System.DateTime.Now.AddMinutes(-3)));
                //clear the queue of any items
                while (DBWorkQueue.DequeueWorkItem(_testHostInstance.HostInstanceID) != null)
                { ; }
                //enqueue the items
                DBWorkQueue.TransactionallyEnqueueWorkScheduleListAndMarkComplete(itemsToEnqueue, scheduleInstances, schedule.ScheduleTypeID, schedule.LastProcessed, System.DateTime.Now.AddSeconds(1), _testHostInstance.HostInstanceID);
                //now try and dequeue the items - should be 3
                DequeueAndAssertWorkItem();
                DequeueAndAssertWorkItem();
                DequeueAndAssertWorkItem();
            }
            finally
            {
                DBJobInstance.DeleteInstanceSchedule(instanceScheduleID1);
                DBJobInstance.DeleteInstanceSchedule(instanceScheduleID2);
                DBJobInstance.DeleteInstanceSchedule(instanceScheduleID3);
                DBJobInstance.DeleteInstanceSchedule(instanceScheduleID4);
                DBJobInstance.DeleteInstanceSchedule(instanceScheduleID5);
            }
        }

        private void DequeueAndAssertWorkItem()
        {
            DequeuedWorkItem deqWI = DBWorkQueue.DequeueWorkItem(_testHostInstance.HostInstanceID);
            Assert.IsNotNull(deqWI);
            Assert.AreEqual(_testJobInstance.JobInstanceID, deqWI.JobInstanceID);
        }

    }


}
