using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using NQueue.BusinessServices.System;
using System.Threading;

namespace NQueue
{
    public partial class CombinedServiceControl : UserControl
    {

        /// <summary>
        /// Delegate to update progress bar control using the same thread in which control was created
        /// </summary>
        private delegate void UpdateProgressBarDelegate(ProgressBar pbBuild, int totalTasks, int tasksCompleted);
        private delegate void UpdateListboxDelegate(ListView list, string message);
        private delegate void UpdateButtonEnablementDelegate(Button button, bool state);

        public CombinedServiceControl()
        {
            InitializeComponent();
        }



        /// <summary>
        /// Writes progress information
        /// </summary>
        /// <param name="progressBar">Progress bar control</param>
        /// <param name="totalTasks">Total tasks enqueued</param>
        /// <param name="tasksCompleted">Total tasks completed</param>
        private static void WriteProgressCalled(ProgressBar progressBar ,int totalTasks, int tasksCompleted)
        {
            if (progressBar.InvokeRequired)
                progressBar.Invoke(new UpdateProgressBarDelegate(UpdateBuildProgressBar), new object[] { progressBar, totalTasks, tasksCompleted });
            else
                UpdateBuildProgressBar(progressBar, totalTasks, tasksCompleted);
        }

        private static void WriteListBoxInfo(ListView list, string message)
        {
            if (list.InvokeRequired)
            {
                list.Invoke(new UpdateListboxDelegate(UpdateListBox), new object[] { list, message });
            }
            else
            {
                UpdateListBox(list, message);
            }
        }


        /// <summary>
        /// Updates progress bar with the specified values.
        /// This method is called by delegate.
        /// </summary>
        /// <param name="progressBar">Progress bar control</param>
        /// <param name="totalTasks">Total tasks to be executed</param>
        /// <param name="tasksCompleted">Total tasks completed</param>
        private static void UpdateBuildProgressBar(ProgressBar progressBar, int totalTasks, int tasksCompleted)
        {
            //Update progressbar with the status
            if(tasksCompleted <= totalTasks)
            progressBar.Value = (int)((float)tasksCompleted / (float)totalTasks * 100);
        }

        private static void UpdateListBox(ListView list, string message)
        {
            ListViewItem newItem = new ListViewItem(new string[] {DateTime.Now.ToString(), message});
            list.Items.Add(newItem);
            newItem.EnsureVisible();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            IssueCommand(ServiceControlCommands.Start);
        }

        private void IssueCommand(ServiceControlCommands command)
        {
            ButtonEnablementChange(false);
            HostInstanceController controller = new HostInstanceController(command);
            controller.InformationMessage += new HostInstanceController.InformationMessageDelegate(controller_InformationMessage);
            controller.Finished += new HostInstanceController.FinishedDelegate(controller_Finished);
            ThreadPool.QueueUserWorkItem(DoAsyncWork, controller);
        }

        private static void DoAsyncWork(object state)
        {
            if (!(state is HostInstanceController)) throw new InvalidOperationException("Async state param was not HostInstanceController");
            ((HostInstanceController)state).ControlHostInstances();
        }

        private void controller_Finished()
        {
            ButtonEnablementChange(true);
        }

        private void controller_InformationMessage(string message, int progress, int size)
        {
            WriteListBoxInfo(lstLog, message);
            WriteProgressCalled(pbProgress, size, progress); 
        }
        private static void CallButtonEnablementChange(Button button, bool state)
        {
            if (button.InvokeRequired)
            {
                button.Invoke(new UpdateButtonEnablementDelegate(ChangeButton), new object[] { button, state });
            }
            else
            {
                ChangeButton(button, state);
            }
        }

        private static void ChangeButton(Button button, bool state)
        {
            button.Enabled = state;
        }
        private void ButtonEnablementChange(bool state)
        {
            CallButtonEnablementChange(btnStart, state);
            CallButtonEnablementChange(btnStop, state);
            CallButtonEnablementChange(btnRestart, state);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            IssueCommand(ServiceControlCommands.Stop);
        }

        private void btnRestart_Click(object sender, EventArgs e)
        {
            IssueCommand(ServiceControlCommands.Restart);
        }
    }
}
