using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Threading;

namespace NQueue
{
    public partial class TypeBrowser : UserControl
    {
        private delegate void RefreshTreeDelegate();
        public delegate void TypeSelectedDelegate(string assemblyFullName, string typeFullName);
        /// <summary>
        /// Fired when a user double clicks a type node in the browser
        /// </summary>
        public event TypeSelectedDelegate TypeSelected;

        public TypeBrowser()
        {
            InitializeComponent();
        }

        private Assembly _asm;
        private bool _displayOnlyConcrete;
        private Type _onlyDisplayInheritorsOf;

        /// <summary>
        /// Displays an assembly in the treeview control
        /// </summary>
        /// <param name="assemblyPath">File path to the assembly to load</param>
        /// <param name="displayOnlyConcrete">If true, abstract classes and interfaces are not displayed</param>
        /// <param name="onlyDisplayInheritorsOf">If not set to null, only those types that inherit from the given type are displayed</param>
        public void DisplayAssembly(string assemblyPath, bool displayOnlyConcrete, Type onlyDisplayInheritorsOf)
        {
            this._asm = Assembly.LoadFrom(assemblyPath);
            this._displayOnlyConcrete = displayOnlyConcrete;
            this._onlyDisplayInheritorsOf = onlyDisplayInheritorsOf;
            //load the tree on its own thread
            treeTypeBrowse.Invoke(new RefreshTreeDelegate(RefreshTree));
        }

        private void RefreshTree()
        {
            //clear the current contents of the tree, if any
            treeTypeBrowse.Nodes.Clear();
            //tree root node is the full strong name of the assembly
            TreeNode rootNode = new TreeNode(_asm.FullName);
            treeTypeBrowse.Nodes.Add(rootNode);

            //iterate over all types in the assembly 
            //keep a lookup of all namespaces already added and where on the treeview they are
            Dictionary<string, TreeNode> namespaceNodes = new Dictionary<string, TreeNode>();
            foreach (Type type in _asm.GetTypes())
            {
                //do not display value types
                if (!type.IsClass) continue; 
                //simply skip all abstract and interface types if instructed to
                if (_displayOnlyConcrete && type.IsAbstract || type.IsInterface) continue;
                //if instructed to, skip all types that don't inherit from the given base class
                if (_onlyDisplayInheritorsOf != null && !type.IsSubclassOf(_onlyDisplayInheritorsOf)) continue;
                //otherwise, add the type to the tree.
                if (!namespaceNodes.ContainsKey(type.Namespace))
                {
                    //if the namespace has not already been added to the tree, do so.
                    TreeNode namespaceNode = new TreeNode(type.Namespace, 0, 0);
                    rootNode.Nodes.Add(namespaceNode);
                    //keep a reference to the treenode, keyed on the namespace name, for later
                    //when we will add leaf nodes under the namespace
                    namespaceNodes.Add(type.Namespace, namespaceNode);
                }
                //add a leaf node to represent the type
                TreeNode typeNode = new TreeNode(type.Name, 1, 1);
                typeNode.Tag = type;    //keep a reference to the actual type object for this type
                namespaceNodes[type.Namespace].Nodes.Add(typeNode);
            }
            rootNode.Expand();
            rootNode.EnsureVisible();
        }

        

        private void OnDoubleClick(object sender, EventArgs e)
        {
            if (treeTypeBrowse.SelectedNode != null)
            {
                if (treeTypeBrowse.SelectedNode.Tag is Type)
                {
                    if (TypeSelected != null) TypeSelected(_asm.FullName,
                        ((Type)treeTypeBrowse.SelectedNode.Tag).FullName);
                }
            }
        }
    }
}
