use NQueue

create table Hosts
(
	HostID int identity(1, 1) not null,
	HostName varchar(70) not null,
	Deleted bit not null default 0,
	constraint PK_HostID primary key clustered (HostID)
)
GO

create table HostInstances
(
	HostInstanceID int identity(1, 1) not null,
	HostID int not null,
	ServerName varchar(256) not null,
	Deleted bit not null default 0,
	constraint PK_HostInstanceID primary key clustered (HostInstanceID),
	constraint FK_HostInstances_HostID foreign key (HostID) references Hosts(HostID)
)
GO

create nonclustered index IXHostInstances_HostID on HostInstances(HostID)
go

create table JobDefinitions
(
	JobDefinitionID int identity(1, 1) not null,
	JobName varchar(70) not null,
	JobDescription varchar(300) null,
	JobAssemblyStrongName nvarchar(256) not null,
	JobClassType nvarchar(256) not null,
	JobConfigFile nvarchar(max) null,
	Deleted bit not null default 0,
	constraint PK_JobDefinitionID primary key clustered (JobDefinitionID)
)
go

create table JobInstanceStates
(
	JobInstanceStateID int not null,
	JobInstanceStateName varchar(60) not null,
	constraint PK_JobInstanceStates primary key clustered (JobInstanceStateID)
)
go

create table JobInstances
(
	JobInstanceID int identity(1, 1) not null,
	JobDefinitionID int not null,
	JobInstanceName varchar(70) not null,
	HostID int not null,
	InstanceTTLTimeSpan int not null default 0,	--Timespan in minutes that an executing job should be allowed to run before being terminated
	JobParameters nvarchar(max) null,	--serialised XML of the parameters object for the job
	JobInstanceStateID int not null default 1,	--enabled, disabled, etc
	Singleton bit not null default 0,			--if 1 can only execute one at a time across the whole group
	Deleted bit not null default 0,
	constraint PK_JobInstanceID primary key clustered (JobInstanceID),
	constraint FK_JobInstance_JobDefinitionID foreign key (JobDefinitionID) references JobDefinitions(JobDefinitionID),
	constraint FK_JobInstance_HostID foreign key (HostID) references Hosts(HostID),
	constraint FK_JobInstance_StateID foreign key (JobInstanceStateID) references JobInstanceStates(JobInstanceStateID)
)
GO

create nonclustered index IX_JobInstancesHostID on JobInstances(HostID)
go

create nonclustered index IX_JobInstancesStateID on JobInstances(JobInstanceStateID)
go


create table WorkItemStatuses
(
	WorkItemStatusID int not null,
	WorkItemStatusDescription varchar(50),
	constraint PK_WorkItemStatusID primary key clustered (WorkItemStatusID)	
)
GO

create table WorkQueue
(
	WorkItemID bigint identity(1, 1) not null,
	HostInstanceID int null,	--instance dealing with the work item - null before being worked on
	JobInstanceID int not null,	--Work to perform
	ServerAffinity varchar(256) null,	--If not null, work can only be dequeued on the given server
	DateEnqueued datetime not null,
	DateToExecute datetime not null,	--work will not be dequeued before this time
	DateDequeued datetime null,
	DateToExpire datetime null,		--unless the job is in a completed or failed state, it should be removed from the queue at this date and time
	DateWorkCompleted datetime null,
	WorkItemStatusID int not null,		--current status of the work item
	constraint PK_WorkItemID primary key clustered (WorkItemID),
	constraint FK_WorkQueue_HostInstanceID foreign key (HostInstanceID) references HostInstances(HostInstanceID),
	constraint FK_WorkQueue_JobInstanceID foreign key (JobInstanceID) references JobInstances(JobInstanceID),
	constraint FK_WorkItemStatusID foreign key (WorkItemStatusID) references WorkItemStatuses(WorkItemStatusID)
)
GO

create nonclustered index IX_WorkQueue_HostIDStatus on WorkQueue(JobInstanceID, WorkItemStatusID, DateToExecute)
go
create nonclustered index IX_WorkQueue_DateToExpire on WorkQueue(DateToExpire)
go


create table WorkLogTypes
(
	WorkLogTypeID int not null,
	WorkLogType varchar(50) not null,		--info, error, warning
	constraint PK_WorkLogTypeID primary key clustered (WorkLogTypeID)
)
GO

create table WorkLog
(
	WorkLogItemID uniqueidentifier not null default NewID(),
	WorkItemID bigint not null,
	WorkLogTypeID int not null,
	LogTime datetime not null default GetDate(),
	StatusDescription nvarchar (1000) not null,
	ExceptionStackTrace nvarchar(max) null,
	constraint PK_WorkLog primary key nonclustered (WorkLogItemID),
	constraint FK_WorkLogTypeID foreign key (WorkLogTypeID) references WorkLogTypes (WorkLogTypeID),
	constraint FK_WorkLogWorkItemID foreign key (WorkItemID) references WorkQueue (WorkItemID)
)
GO
create clustered index IX_WorkLogOrder on WorkLog(WorkItemID, LogTime)
create nonclustered index IX_WorkLogItemID on WorkLog(WorkItemID)
go


create table ScheduleTypes
(
	ScheduleTypeID int not null,
	ScheduleType varchar(50) not null,
	CurrentlyProcessing bit not null default 0,
	LastProcessed datetime null,
	NextProcessDue datetime null,
	ProcessingHostInstanceID int null,
	ScheduleTypeStrongName nvarchar(256) not null,
	ScheduleTypeClassType nvarchar(256) not null,
	constraint PK_ScheduledTypeQueue primary key clustered (ScheduleTypeID),
	constraint FK_ScheduledTypeQueue_HostInstanceID foreign key (ProcessingHostInstanceID) references HostInstances(HostInstanceID)
)
GO

create table JobInstanceSchedule
(
	JobInstanceScheduleID int identity(1, 1) not null,
	JobInstanceID int not null,
	ScheduleTypeID int not null,
	InstanceScheduleLastRun datetime null,
	ScheduleParameters nvarchar(max) not null,	--xml of the serialised parameters object for the schedule
	Deleted bit not null default 0,
	constraint PK_JobInstanceSchedule primary key clustered (JobInstanceScheduleID),
	constraint FK_JobInstanceSchedule_JobInstanceID foreign key (JobInstanceID) references JobInstances(JobInstanceID),
	constraint FK_JobInstanceSchedule_ScheduleTypeID foreign key (ScheduleTypeID) references ScheduleTypes(ScheduleTypeID)
)
GO

create nonclustered index IX_JobInstanceScheduleTypeID on JobInstanceSchedule(ScheduleTypeID)
go


create table Mutex
(
	MutexID int identity(1, 1) not null,
	MutexName varchar(500) not null,
	ServerWithLock varchar(256) null,
	ExpiryDate datetime not null
	constraint PK_Mutex_MutexID primary key clustered (MutexID)
)
GO
create unique nonclustered index IX_Mutex_MutexName on Mutex(MutexName)
create nonclustered index IX_Mutex_ExpiryDate on Mutex(ExpiryDate)
go



----Autogenerated site map tables

/****** Object:  Table [dbo].[AspNet_SqlCacheTablesForChangeNotification]    Script Date: 06/13/2008 14:24:28 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[AspNet_SqlCacheTablesForChangeNotification](
	[tableName] [nvarchar](450) NOT NULL,
	[notificationCreated] [datetime] NOT NULL DEFAULT (getdate()),
	[changeId] [int] NOT NULL DEFAULT ((0)),
PRIMARY KEY CLUSTERED 
(
	[tableName] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO


